-- NOTE: I do not play bst, so this will not be maintained for 'active' use. 
-- It is added to the repository to allow people to have a baseline to build from,
-- and make sure it is up-to-date with the library API.

-- Credit to Quetzalcoatl.Falkirk for most of the original work.

--[[
    Custom commands:
    
    Ctrl-F8 : Cycle through available pet food options.
    Alt-F8 : Cycle through correlation modes for pet attacks.
]]

-------------------------------------------------------------------------------------------------------------------
-- Setup functions for this job.  Generally should not be modified.
-------------------------------------------------------------------------------------------------------------------

-- Initialization function for this job file.
function get_sets()
    mote_include_version = 2
    
    -- Load and initialize the include file.
    include('Mote-Include.lua')
	set_language('japanese')
end

function job_setup()
    -- Set up Reward Modes and keybind Ctrl-F8
    state.RewardMode = M{['description']='Reward Mode', 'Theta', 'Zeta', 'Eta'}
    RewardFood = {name="ペットシータ"}
    --send_command('bind ^f8 gs c cycle RewardMode')

    -- Set up Monster Correlation Modes and keybind Alt-F8
    state.CorrelationMode = M{['description']='Correlation Mode', 'Neutral','Favorable'}
    --send_command('bind !f8 gs c cycle CorrelationMode')
    
    -- Custom pet modes for engaged gear
    state.PetMode = M{['description']='Pet Mode', 'Normal', 'PetStance', 'PetTank'}


    ready_moves_to_check = S{
		'ほんきだせ',
		'フットキック','土煙','爪旋風脚','ワイルドカロット','雪煙',
		'頭突き','レイジ','シープチャージ','シープソング',
		'咆哮','レイザーファング','クローサイクロン','クロススラッシュ','プレダトリグレア',
		'カオティックアイ','ブラスター','チャージドホイスカー','怒髪',
		'スイープガウジ','ゼラススノート',
		'ファイアボール','テイルブロー','ブロックヘッド','ブレインクラッシュ','体液分泌','超低周波',
		'ガイストウォール','ナビングノイズ','ニンブルスナップ','サイクロテール','トクシックスピット',
		'サイズテール','リッパーファング','噛みつきラッシュ',
		'トータスストンプ','甲羅強化','アクアブレス',
		'カースドスフィア','毒液','サマーソルト',
		'パワーアタック','高周波フィールド','ライノアタック','ライノガード','スポイル','ライノレッカー',
		'サンドブラスト','サンドピット','ベノムスプレー','マンディビュラバイト',
		'ダブルクロー','グラップル','スピニングトップ','フィラメンテッドホールド',
		'サドンランジ','スパイラルスピン','ノイサムパウダー',
		'センシラブレード','テグミナバフェット',
		'シックルスラッシュ','アシッドスプレー','スパイダーウェブ',
		'インフェクティドサッカー','グルームスプレー',
		'ディスエンバウエル','エクティアパーチングサルヴォ',
        'ヘッドバット','夢想花','種まき','リーフダガー','スクリーム',
		'フロッグキック','スポア','マヨイタケ','シビレタケ','オドリタケ','サイレスガス','ダークスポア',
		'ニードルショット','針？本',
        'サペリフィック','グロオーサケス','パルジィパレン',
		'ティックルテンドリル','スティンクボム','ネクターデルージュ','ネペンシックプランジ',
        'ウィングスラップ','ビークランジ',
		'バックヒール','ジェタチュラ','チョークブレス','ファンタッド','フーフボレー','ニヒリティソング',
		'モルトプルメイジ','スワープフレンジー','ペンタペック',
		'ペッキングフラリー',
		'バブルシャワー','バブルカーテン','ビッグシザー','シザーガード','メタルボディ','ベノムシャワー','メガシザース',
		'威嚇','リコイルダイブ','ウォーターウォール',
        '吸着','ドレインキッス','アシッドミスト','TP吸収キッス',
        'ピュルラントウーズ','コローシブウーズ',
        'ファウルウォーター','ペステレントプルーム',
		'フルイドスルー','フルイドスプレッド','消化',
		}
end

-------------------------------------------------------------------------------------------------------------------
-- User setup functions for this job.  Recommend that these be overridden in a sidecar file.
-------------------------------------------------------------------------------------------------------------------

function user_setup()
    state.OffenseMode:options('Normal', 'Acc')
    state.WeaponskillMode:options('Normal', 'Acc')
    state.IdleMode:options('Normal', 'Refresh', 'Reraise')
    state.PhysicalDefenseMode:options('PDT', 'Hybrid', 'Killer')

    update_combat_form()
end


-- Called when this job file is unloaded (eg: job change)
function user_unload()
    -- Unbinds the Reward and Correlation hotkeys.
    --send_command('unbind ^f8')
    --send_command('unbind !f8')
end


-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for standard casting events.
-------------------------------------------------------------------------------------------------------------------

function job_precast(spell, action, spellMap, eventArgs)
    -- Define class for Sic and Ready moves.
    if ready_moves_to_check:contains(spell.name) and pet.status == 'Engaged' then
        classes.CustomClass = "WS"
	equip(sets.precast.Ready)
    end
end


function job_post_precast(spell, action, spellMap, eventArgs)
    -- If Killer Instinct is active during WS, equip Ferine Gausape +2.
    if spell.type:lower() == 'weaponskill' and buffactive['K.インスティンクト'] then
        equip(sets.buff['K.インスティンクト'])
    end
end


function job_pet_post_midcast(spell, action, spellMap, eventArgs)
    -- Equip monster correlation gear, as appropriate
    equip(sets.midcast.Pet[state.CorrelationMode.value])
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for non-casting events.
-------------------------------------------------------------------------------------------------------------------

function job_buff_change(buff, gain)
    if buff == 'K.インスティンクト' then
        update_combat_form()
        handle_equipping_gear(player.status)
    end
end

-- Called when the pet's status changes.
function job_pet_status_change(newStatus, oldStatus)

end

-- Handle notifications of general user state change.
function job_state_change(stateField, newValue, oldValue)
    if stateField == 'Reward Mode' then
        -- Thena, Zeta or Eta
        RewardFood.name = "Pet Food " .. newValue
    elseif stateField == 'Pet Mode' then
        state.CombatWeapon:set(newValue)
    end
end

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements standard library decisions.
-------------------------------------------------------------------------------------------------------------------

-- Called by the 'update' self-command, for common needs.
-- Set eventArgs.handled to true if we don't want automatic equipping of gear.
function job_update(cmdParams, eventArgs)
    update_combat_form()
end


-- Set eventArgs.handled to true if we don't want the automatic display to be run.
function display_current_job_state(eventArgs)
    local msg = 'Melee'
    
    if state.CombatForm.has_value then
        msg = msg .. ' (' .. state.CombatForm.value .. ')'
    end
    
    msg = msg .. ': '
    
    msg = msg .. state.OffenseMode.value
    if state.HybridMode.value ~= 'Normal' then
        msg = msg .. '/' .. state.HybridMode.value
    end
    msg = msg .. ', WS: ' .. state.WeaponskillMode.value
    
    if state.DefenseMode.value ~= 'None' then
        msg = msg .. ', ' .. 'Defense: ' .. state.DefenseMode.value .. ' (' .. state[state.DefenseMode.value .. 'DefenseMode'].value .. ')'
    end
    
    if state.Kiting.value then
        msg = msg .. ', Kiting'
    end

    msg = msg .. ', Reward: '..state.RewardMode.value..', Correlation: '..state.CorrelationMode.value

    add_to_chat(122, msg)

    eventArgs.handled = true
end


-------------------------------------------------------------------------------------------------------------------
-- Utility functions specific to this job.
-------------------------------------------------------------------------------------------------------------------

function update_combat_form()
    if buffactive['K.インスティンクト'] then
        if (player.sub_job == '忍' or player.sub_job == '踊') and player.equipment.sub:endswith('Axe') then
            state.CombatForm:set('KillerDW')
        else
            state.CombatForm:set('Killer')
        end
    elseif (player.sub_job == '忍' or player.sub_job == '踊') and player.equipment.sub:endswith('Axe') then
        state.CombatForm:set('DW')
    else
        state.CombatForm:reset()
    end
end


