-------------------------------------------------------------------------------------------------------------------
-- Setup functions for this job.  Generally should not be modified.
-------------------------------------------------------------------------------------------------------------------

--[[
    gs c toggle LuzafRing -- Toggles use of Luzaf Ring on and off
    
    Offense mode is melee or ranged.  Used ranged offense mode if you are engaged
    for ranged weaponskills, but not actually meleeing.
    
    Weaponskill mode, if set to 'Normal', is handled separately for melee and ranged weaponskills.
--]]


-- Initialization function for this job file.
function get_sets()
    mote_include_version = 2
    
    -- Load and initialize the include file.
    include('Mote-Include.lua')
	set_language('japanese')
end

-- Setup vars that are user-independent.  state.Buff vars initialized here will automatically be tracked.
function job_setup()
    -- Whether to use Luzaf's Ring
    state.LuzafRing = M(false, "Luzaf's Ring")
    -- Whether a warning has been given for low ammo
    state.warned = M(false)

    define_roll_values()
end

-------------------------------------------------------------------------------------------------------------------
-- User setup functions for this job.  Recommend that these be overridden in a sidecar file.
-------------------------------------------------------------------------------------------------------------------

-- Setup vars that are user-dependent.  Can override this function in a sidecar file.
function user_setup()
    state.OffenseMode:options('Ranged', 'Melee', 'Acc')
    state.RangedMode:options('Normal', 'Acc')
    state.WeaponskillMode:options('Normal', 'Acc', 'Att', 'Mod')
    state.CastingMode:options('Normal', 'Resistant')
    state.IdleMode:options('Normal', 'PDT', 'Refresh')

    gear.RAbullet = "デベステトブレット"
    gear.WSbullet = "デベステトブレット"
    gear.MAbullet = "ライヴブレット"
    gear.QDbullet = "ライヴブレット"
    options.ammo_warning_limit = 15
    Card_warning_limit = 15   

    -- Additional local binds
    --send_command('bind ^` input /ja "Double-up" <me>')
    --send_command('bind !` input /ja "Bolter\'s Roll" <me>')

    --select_default_macro_book()
end


-- Called when this job file is unloaded (eg: job change)
function user_unload()
    --send_command('unbind ^`')
    --send_command('unbind !`')
end


-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for standard casting events.
-------------------------------------------------------------------------------------------------------------------

-- Set eventArgs.handled to true if we don't want any automatic gear equipping to be done.
-- Set eventArgs.useMidcastGear to true if we want midcast gear equipped on precast.
function job_precast(spell, action, spellMap, eventArgs)
    -- Check that proper ammo is available if we're using ranged attacks or similar.
    if spell.action_type == 'Ranged Attack' or spell.type == 'WeaponSkill' or spell.type == 'CorsairShot' then
        do_bullet_checks(spell, spellMap, eventArgs)
    end

    -- gear sets
    if (spell.type == 'CorsairRoll' or spell.name == "ダブルアップ") and state.LuzafRing.value then
        equip(sets.precast.LuzafRing)
    elseif spell.type == 'CorsairShot' and state.CastingMode.value == 'Resistant' then
        classes.CustomClass = 'Acc'
    elseif spell.name == 'フォールド' and buffactive['バスト'] == 2 then
        if sets.precast.FoldDoubleBust then
            equip(sets.precast.FoldDoubleBust)
            eventArgs.handled = true
        end
    end
end


-- Set eventArgs.handled to true if we don't want any automatic gear equipping to be done.
function job_aftercast(spell, action, spellMap, eventArgs)
    if spell.type == 'CorsairRoll' and not spell.interrupted then
        display_roll_info(spell)
    end
end

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements standard library decisions.
-------------------------------------------------------------------------------------------------------------------


-- Called by the 'update' self-command, for common needs.
-- Set eventArgs.handled to true if we don't want automatic equipping of gear.
function job_update(cmdParams, eventArgs)
    if newStatus == 'Engaged' and player.equipment.main == 'チャトヤンスタッフ' then
        state.OffenseMode:set('Ranged')
    end
end


-- Set eventArgs.handled to true if we don't want the automatic display to be run.
function display_current_job_state(eventArgs)
    local msg = ''
    
    msg = msg .. 'Off.: '..state.OffenseMode.current
    msg = msg .. ', Rng.: '..state.RangedMode.current
    msg = msg .. ', WS.: '..state.WeaponskillMode.current
    msg = msg .. ', QD.: '..state.CastingMode.current

    if state.DefenseMode.value ~= 'None' then
        local defMode = state[state.DefenseMode.value ..'DefenseMode'].current
        msg = msg .. ', Defense: '..state.DefenseMode.value..' '..defMode
    end
    
    if state.Kiting.value then
        msg = msg .. ', Kiting'
    end
    
    if state.PCTargetMode.value ~= 'default' then
        msg = msg .. ', Target PC: '..state.PCTargetMode.value
    end

    if state.SelectNPCTargets.value then
        msg = msg .. ', Target NPCs'
    end

    msg = msg .. ', Roll Size: ' .. ((state.LuzafRing.value and 'Large') or 'Small')
    
    add_to_chat(122, msg)

    eventArgs.handled = true
end


-------------------------------------------------------------------------------------------------------------------
-- Utility functions specific to this job.
-------------------------------------------------------------------------------------------------------------------

function define_roll_values()
    rolls = {
        ["コルセアズロール"]   = {lucky=5, unlucky=9, bonus="取得経験値量"},
        ["ニンジャロール"]       = {lucky=4, unlucky=8, bonus="回避"},
        ["ハンターズロール"]    = {lucky=4, unlucky=8, bonus="命中・飛命"},
        ["カオスロール"]       = {lucky=4, unlucky=8, bonus="攻撃力・飛攻"},
        ["メガスズロール"]     = {lucky=2, unlucky=6, bonus="魔法防御力"},
        ["ヒーラーズロール"]    = {lucky=3, unlucky=7, bonus="	被ケアル回復量"},
        ["パペットロール"]      = {lucky=4, unlucky=8, bonus="ペット魔法攻撃力・魔法命中率"},
        ["コーラルロール"]      = {lucky=2, unlucky=6, bonus="詠唱中断率ダウン"},
        ["モンクスロール"]      = {lucky=3, unlucky=7, bonus="モクシャアップ"},
        ["ビーストロール"]       = {lucky=4, unlucky=8, bonus="ペット攻撃力"},
        ["サムライロール"]     = {lucky=2, unlucky=6, bonus="ストアTP"},
        ["エボカーズロール"]    = {lucky=5, unlucky=9, bonus="リフレシュ"},
        ["ローグズロール"]     = {lucky=5, unlucky=9, bonus="クリティカルヒット確率"},
        ["ワーロックスロール"]   = {lucky=4, unlucky=8, bonus="魔法命中"},
        ["ファイターズロール"]   = {lucky=5, unlucky=9, bonus="ダブルアタック効果"},
        ["ドラケンロール"]     = {lucky=3, unlucky=7, bonus="ペット命中・飛命"},
        ["ガランツロール"]   = {lucky=3, unlucky=7, bonus="防御力"},
        ["ウィザーズロール"]    = {lucky=5, unlucky=9, bonus="魔法攻撃力"},
        ["ダンサーロール"]    = {lucky=3, unlucky=7, bonus="リジェネ"},
        ["スカラーロール"]   = {lucky=2, unlucky=6, bonus="コンサーブMP"},
        ["ボルターズロール"]    = {lucky=3, unlucky=9, bonus="移動速度"},
        ["キャスターズロール"]    = {lucky=2, unlucky=7, bonus="ファストキャスト"},
        ["コアサーズロール"]   = {lucky=3, unlucky=9, bonus="スナップショット"},
        ["ブリッツァロール"]   = {lucky=4, unlucky=9, bonus="攻撃間隔"},
        ["タクティックロール"] = {lucky=5, unlucky=8, bonus="リゲイン・リジェネ"},
        ["アライズロール"]    = {lucky=3, unlucky=10, bonus="連携ダメージ・命中"},
        ["マイザーロール"]     = {lucky=5, unlucky=7, bonus="セーブTP"},
        ["コンパニオンロール"] = {lucky=2, unlucky=10, bonus="ペットリゲイン・リジェネ"},
        ["カウンターロール"]   = {lucky=4, unlucky=8, bonus="カウンター確率"},
    }
end

function display_roll_info(spell)
    rollinfo = rolls[spell.english]
    local rollsize = (state.LuzafRing.value and 'Large') or 'Small'

    if rollinfo then
        add_to_chat(104, spell.japanese..' '..rollinfo.bonus..'.  ロール: '..rollsize)
         add_to_chat(104, 'ラッキーNO.'..tostring(rollinfo.lucky)..',アンラッキーNO.'..tostring(rollinfo.unlucky)..'.')
    end
end


-- Determine whether we have sufficient ammo for the action being attempted.
function do_bullet_checks(spell, spellMap, eventArgs)
    local bullet_name
    local bullet_min_count = 1
    
    if spell.type == 'WeaponSkill' then
        if spell.skill == "Marksmanship" then
            if spell.element == 'None' then
                -- physical weaponskills
                bullet_name = gear.WSbullet
            else
                -- magical weaponskills
                bullet_name = gear.MAbullet
            end
        else
            -- Ignore non-ranged weaponskills
            return
        end
    elseif spell.type == 'CorsairShot' then
        bullet_name = gear.QDbullet
    elseif spell.action_type == 'Ranged Attack' then
        bullet_name = gear.RAbullet
        if buffactive['トリプルショット'] then
            bullet_min_count = 3
        end
    end
    
    local available_bullets = player.inventory[bullet_name] or player.wardrobe[bullet_name]
    
    -- If no ammo is available, give appropriate warning and end.
    if not available_bullets then
        if spell.type == 'CorsairShot' and player.equipment.ammo ~= 'empty' then
            add_to_chat(104, 'No Quick Draw ammo left.  Using what\'s currently equipped ('..player.equipment.ammo..').')
            return
        elseif spell.type == 'WeaponSkill' and player.equipment.ammo == gear.RAbullet then
            add_to_chat(104, 'No weaponskill ammo left.  Using what\'s currently equipped (standard ranged bullets: '..player.equipment.ammo..').')
            return
        else
            add_to_chat(104, 'No ammo ('..tostring(bullet_name)..') available for that action.')
            eventArgs.cancel = true
            return
        end
    end
    
    -- Don't allow shooting or weaponskilling with ammo reserved for quick draw.
    if spell.type ~= 'CorsairShot' and bullet_name == gear.QDbullet and available_bullets.count <= bullet_min_count then
        add_to_chat(104, 'No ammo will be left for Quick Draw.  Cancelling.')
        eventArgs.cancel = true
        return
    end
    
    -- Low ammo warning.
    if spell.type ~= 'CorsairShot' 
        and available_bullets.count > 1 and available_bullets.count <= options.ammo_warning_limit then
        local msg = '     =warning= '..bullet_name..' 残弾 '..tostring(available_bullets.count)..'発 =warning=     '
        --local border = string.repeat("*", #msg)
        local border = ""
        for i = 17, #msg do
            border = border .. "*"
        end
        add_to_chat(104, border)
        add_to_chat(104, msg)
        add_to_chat(104, border)	
        state.warned:set()
    elseif available_bullets.count > options.ammo_warning_limit and state.warned then
        state.warned:reset()
	end

	if spell.type == 'CorsairShot' then
			if player.inventory['エレメントカード'].count <= Card_warning_limit then
				local msg = '     =warning= エレメントカード 残 '..tostring(player.inventory['エレメントカード'].count-1)..'枚 =warning=     '
				local border = ""
				for i = 17, #msg do
				border = border .. "="
				end
				add_to_chat(104, border)
				add_to_chat(104, msg)
				add_to_chat(104, border)
				return
			end
			return
	end
end

-- Select default macro book on initial load or subjob change.
function select_default_macro_book()
    set_macro_page(1, 19)
end