-------------------------------------------------------------------------------------------------------------------
-- Setup functions for this job.  Generally should not be modified.
-------------------------------------------------------------------------------------------------------------------

-- Initialization function for this job file.
function get_sets()
    mote_include_version = 2
    
    -- Load and initialize the include file.
    include('Mote-Include.lua')
	set_language('japanese')
end


-- Setup vars that are user-independent.  state.Buff vars initialized here will automatically be tracked.
function job_setup()
	state.Buff['乱れ撃ち']= buffactive['乱れ撃ち'] or false
	state.Buff['カモフラージュ']= buffactive['カモフラージュ'] or false
	state.Buff['エンドレスショット'] = buffactive['エンドレスショット'] or false
end

-------------------------------------------------------------------------------------------------------------------
-- User setup functions for this job.  Recommend that these be overridden in a sidecar file.
-------------------------------------------------------------------------------------------------------------------

-- Setup vars that are user-dependent.  Can override this function in a sidecar file.
function user_setup()
    state.OffenseMode:options('Normal', 'Acc')
	state.RangedMode:options('Normal', 'Acc')
	state.WeaponskillMode:options('Normal', 'Acc')
	
	gear.default.weaponskill_neck = "Ocachi Gorget"
	gear.default.weaponskill_waist = "Elanid Belt"
	
	DefaultAmmo = {['ガーンデーヴァ'] = "アルテミスの矢", ['ガストラフェテス'] = "クェリングボルト",['アルマゲドン'] = "エラデケトブレット"}
	U_Shot_Ammo = {['ガーンデーヴァ'] = "アルテミスの矢", ['ガストラフェテス'] = "クェリングボルト",['アルマゲドン'] = "エラデケトブレット"} 

	--select_default_macro_book()

	--send_command('bind f9 gs c cycle RangedMode')
	--send_command('bind ^f9 gs c cycle OffenseMode')
end


-- Called when this job file is unloaded (eg: job change)
function user_unload()
	--send_command('unbind f9')
	--send_command('unbind ^f9')
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for standard casting events.
-------------------------------------------------------------------------------------------------------------------

-- Set eventArgs.handled to true if we don't want any automatic gear equipping to be done.
-- Set eventArgs.useMidcastGear to true if we want midcast gear equipped on precast.
function job_precast(spell, action, spellMap, eventArgs)
	if spell.action_type == 'Ranged Attack' then
		state.CombatWeapon:set(player.equipment.range)
	end

	if spell.action_type == 'Ranged Attack' or
	  (spell.type == 'WeaponSkill' and (spell.skill == '射撃' or spell.skill == '弓術')) then
		check_ammo(spell, action, spellMap, eventArgs)
	end
	
	if state.DefenseMode.value ~= 'None' and spell.type == 'WeaponSkill' then
		-- Don't gearswap for weaponskills when Defense is active.
		eventArgs.handled = true
	end
end


-- Set eventArgs.handled to true if we don't want any automatic gear equipping to be done.
function job_midcast(spell, action, spellMap, eventArgs)
	if spell.action_type == 'Ranged Attack' and state.Buff['乱れ撃ち'] then
		equip(sets.buff['乱れ撃ち'])
		eventArgs.handled = true
	end
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for non-casting events.
-------------------------------------------------------------------------------------------------------------------

-- Called when a player gains or loses a buff.
-- buff == buff gained or lost
-- gain == true if the buff was gained, false if it was lost.
function job_buff_change(buff, gain)
	if buff == "カモフラージュ" then
		if gain then
			equip(sets.Buff['カモフラージュ'])
			disable('body')
		else
			enable('body')
		end
	end
end

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements standard library decisions.
-------------------------------------------------------------------------------------------------------------------

-- Set eventArgs.handled to true if we don't want the automatic display to be run.
function display_current_job_state(eventArgs)

end

-------------------------------------------------------------------------------------------------------------------
-- Utility functions specific to this job.
-------------------------------------------------------------------------------------------------------------------

-- Check for proper ammo when shooting or weaponskilling
function check_ammo(spell, action, spellMap, eventArgs)

	local available_U_Shot_Ammo =
		player.inventory[U_Shot_Ammo[player.equipment.range]] or
		player.wardrobe[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe2[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe3[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe4[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe5[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe6[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe7[U_Shot_Ammo[player.equipment.range]] or 
		player.wardrobe8[U_Shot_Ammo[player.equipment.range]]

	local available_DefaultAmmo =
		player.inventory[DefaultAmmo[player.equipment.range]] or
		player.wardrobe[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe2[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe3[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe4[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe5[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe6[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe7[DefaultAmmo[player.equipment.range]] or 
		player.wardrobe8[DefaultAmmo[player.equipment.range]]

    local available_ammo =
		player.inventory[player.equipment.ammo] or 
		player.wardrobe[player.equipment.ammo] or 
		player.wardrobe2[player.equipment.ammo] or 
		player.wardrobe3[player.equipment.ammo] or 
		player.wardrobe4[player.equipment.ammo] or 
		player.wardrobe5[player.equipment.ammo] or 
		player.wardrobe6[player.equipment.ammo] or 
		player.wardrobe7[player.equipment.ammo] or 
		player.wardrobe8[player.equipment.ammo]

	-- Filter ammo checks depending on Unlimited Shot
	if state.Buff['エンドレスショット'] then
		if player.equipment.ammo ~= U_Shot_Ammo[player.equipment.range] then
			if available_U_Shot_Ammo then
				add_to_chat(122,"エンドレスショット:有効 → エンドレスショットに設定した矢弾を装備します")
				equip({ammo=U_Shot_Ammo[player.equipment.range]})
			else
				add_to_chat(122,"エンドレスショット:有効 → しかしエンドレスショットに設定した矢弾が見つかりません")
			end
		end
	else
		if player.equipment.ammo == U_Shot_Ammo[player.equipment.range] and player.equipment.ammo ~= DefaultAmmo[player.equipment.range] then
			if DefaultAmmo[player.equipment.range] then
				if available_DefaultAmmo then
					add_to_chat(122,"エンドレスショット:無効 → デフォルトに設定した矢弾を装備します")
					equip({ammo=DefaultAmmo[player.equipment.range]})
				else
					add_to_chat(122,"エンドレスショット:無効 → エンドレスショットに設定した矢弾を装備解除します")
					equip({ammo=empty})
				end
			else
				add_to_chat(122,"デフォルトの弾が見つからないのでエンドレスショットに設定した矢弾を装備解除します")
				equip({ammo=empty})
			end
		elseif player.equipment.ammo == 'empty' then
			if DefaultAmmo[player.equipment.range] then
				if available_DefaultAmmo then
					add_to_chat(122,"デフォルトに設定した矢弾を装備します")
					equip({ammo=DefaultAmmo[player.equipment.range]})
				else
					add_to_chat(122,"デフォルトに設定した矢弾を使用出来ないので矢弾を装備しません")
				end
			else
				add_to_chat(122,"現在の武器のデフォルトの矢弾が設定されていないため矢弾を装備しません")
			end
		elseif available_ammo.count < 99 then
			add_to_chat(122,""..available_ammo.shortname.." 残弾数 ("..available_ammo.count..") 発")
		end
	end
end



-- Select default macro book on initial load or subjob change.
function select_default_macro_book()
	set_macro_page(1, 7)
end

