-------------------------------------------------------------------------------------------------------------------
-- Setup functions for this job.  Generally should not be modified.
-------------------------------------------------------------------------------------------------------------------

--[[
        Custom commands:

        Shorthand versions for each strategem type that uses the version appropriate for
        the current Arts.

                                        白のグリモア              黒のグリモア

        gs c scholar light              白のグリモア/白の補遺
        gs c scholar dark                                      黒のグリモア/黒の補遺
        gs c scholar cost               簡素清貧の章              勤倹小心の章
        gs c scholar speed              電光石火の章              疾風迅雷の章
        gs c scholar aoe                女神降臨の章              精霊光来の章
        gs c scholar power              意気昂然の章              気炎万丈の章
        gs c scholar duration           令狸執鼠の章
        gs c scholar accuracy           不惜身命の章              一心精進の章
        gs c scholar enmity             天衣無縫の章              無憂無風の章
        gs c scholar skillchain                                震天動地の章
        gs c scholar addendum           白の補遺                 黒の補遺
--]]



-- Initialization function for this job file.
function get_sets()
    mote_include_version = 2
    
    -- Load and initialize the include file.
    include('Mote-Include.lua')
	set_language('japanese')
end

-- Setup vars that are user-independent.  state.Buff vars initialized here will automatically be tracked.
function job_setup()
    info.addendumNukes = S{"ストーンIV", "ウォータIV", "エアロIV", "ファイアIV", "ブリザドIV", "サンダーIV",
        "ストーンV", "ウォータV", "エアロV", "ファイアV", "ブリザドV", "サンダーV"}

    state.Buff['机上演習:蓄積中'] = buffactive['机上演習:蓄積中'] or false
    update_active_strategems()
end

-------------------------------------------------------------------------------------------------------------------
-- User setup functions for this job.  Recommend that these be overridden in a sidecar file.
-------------------------------------------------------------------------------------------------------------------

-- Setup vars that are user-dependent.  Can override this function in a sidecar file.
function user_setup()
    state.OffenseMode:options('Normal', 'Acc')
    state.OffenseMode:options('None', 'Normal')
    state.CastingMode:options('Normal', 'Resistant')
    state.IdleMode:options('Normal', 'PDT')


    info.low_nukes = S{"ストーン", "ウォータ", "エアロ", "ファイア", "ブリザド", "サンダー"}
    info.mid_nukes = S{"ストーンII", "ウォータII", "エアロII", "ファイアII", "ブリザドII", "サンダーII",
                        "ストーンIII", "ウォータIII", "エアロIII", "ファイアIII", "ブリザドIII", "サンダーIII",
                        "ストーンIV", "ウォータIV", "エアロIV", "ファイアIV", "ブリザドIV", "サンダーIV",}
    info.high_nukes = S{"ストーンV", "ウォータV", "エアロV", "ファイアV", "ブリザドV", "サンダーV"}

    --send_command('bind ^` input /ma Stun <t>')

    --select_default_macro_book()
end

function user_unload()
    --send_command('unbind ^`')
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for standard casting events.
-------------------------------------------------------------------------------------------------------------------

-- Run after the general midcast() is done.
function job_post_midcast(spell, action, spellMap, eventArgs)
    if spell.action_type == 'Magic' then
        apply_grimoire_bonuses(spell, action, spellMap, eventArgs)
    end
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for non-casting events.
-------------------------------------------------------------------------------------------------------------------

-- Called when a player gains or loses a buff.
-- buff == buff gained or lost
-- gain == true if the buff was gained, false if it was lost.
function job_buff_change(buff, gain)
    if buff == "机上演習:蓄積中" then
        handle_equipping_gear(player.status)
    end
end

-- Handle notifications of general user state change.
--[[
-- Handle notifications of general user state change.
function job_state_change(stateField, newValue, oldValue)
    if stateField == 'Offense Mode' then
        if newValue == 'Normal' then
            disable('main','sub','range')
        else
            enable('main','sub','range')
        end
    end
end
]]

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements standard library decisions.
-------------------------------------------------------------------------------------------------------------------

-- Custom spell mapping.
function job_get_spell_map(spell, default_spell_map)
    if spell.action_type == 'Magic' then
        if default_spell_map == 'Cure' or default_spell_map == 'Curaga' then
            if world.weather_element == 'Light' then
                return 'CureWithLightWeather'
            end
        elseif spell.skill == '弱体魔法' then
            if spell.type == '白魔法' then
                return 'MndEnfeebles'
            else
                return 'IntEnfeebles'
            end
        elseif spell.skill == '精霊魔法' then
            if info.low_nukes:contains(spell.english) then
                return 'LowTierNuke'
            elseif info.mid_nukes:contains(spell.english) then
                return 'MidTierNuke'
            elseif info.high_nukes:contains(spell.english) then
                return 'HighTierNuke'
            end
        end
    end
end

function customize_idle_set(idleSet)
    if state.Buff['机上演習:蓄積中'] then
        if state.IdleMode.value == 'Normal' then
            idleSet = set_combine(idleSet, sets.buff.FullSublimation)
        elseif state.IdleMode.value == 'PDT' then
            idleSet = set_combine(idleSet, sets.buff.PDTSublimation)
        end
    end

    if player.mpp < 51 then
        idleSet = set_combine(idleSet, sets.latent_refresh)
    end

    return idleSet
end

-- Called by the 'update' self-command.
function job_update(cmdParams, eventArgs)
    if cmdParams[1] == 'user' and not (buffactive['白のグリモア']      or buffactive['黒のグリモア'] or
                        buffactive['白の補遺'] or buffactive['黒の補遺']) then
        if state.IdleMode.value == 'Stun' then
            send_command('@input /ja '..windower.to_shift_jis("黒のグリモア")..' <me>')
        else
            send_command('@input /ja '..windower.to_shift_jis("白のグリモア")..' <me>')
        end
    end

    update_active_strategems()
    update_sublimation()
end

-- Function to display the current relevant user state when doing an update.
-- Return true if display was handled, and you don't want the default info shown.
function display_current_job_state(eventArgs)
    display_current_caster_state()
    eventArgs.handled = true
end

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements self-commands.
-------------------------------------------------------------------------------------------------------------------

-- Called for direct player commands.
function job_self_command(cmdParams, eventArgs)
    if cmdParams[1]:lower() == 'scholar' then
        handle_strategems(cmdParams)
        eventArgs.handled = true
    end
end

-------------------------------------------------------------------------------------------------------------------
-- Utility functions specific to this job.
-------------------------------------------------------------------------------------------------------------------

-- Reset the state vars tracking strategems.
function update_active_strategems()
    state.Buff['気炎万丈の章'] = buffactive['気炎万丈の章'] or false
    state.Buff['意気昂然の章'] = buffactive['意気昂然の章'] or false
    state.Buff['令狸執鼠の章'] = buffactive['令狸執鼠の章'] or false
    state.Buff['震天動地の章'] = buffactive['震天動地の章'] or false
    state.Buff['簡素清貧の章'] = buffactive['簡素清貧の章'] or false
    state.Buff['勤倹小心の章'] = buffactive['勤倹小心の章'] or false
    state.Buff['電光石火の章'] = buffactive['電光石火の章'] or false
    state.Buff['疾風迅雷の章'] = buffactive['疾風迅雷の章'] or false

    state.Buff['虚誘掩殺の策'] = buffactive['虚誘掩殺の策'] or false
end

function update_sublimation()
    state.Buff['机上演習:蓄積中'] = buffactive['机上演習:蓄積中'] or false
end

-- Equip sets appropriate to the active buffs, relative to the spell being cast.
function apply_grimoire_bonuses(spell, action, spellMap)
    if state.Buff['令狸執鼠の章'] and spell.type =='白魔法' and spell.skill == '強化魔法' then
        equip(sets.buff['令狸執鼠の章'])
    end
    if state.Buff['意気昂然の章'] and (spellMap == 'Cure' or spellMap == 'Curaga') then
        equip(sets.buff['意気昂然の章'])
    end
    if spell.skill == '精霊魔法' and spellMap ~= 'ElementalEnfeeble' then
        if state.Buff['気炎万丈の章'] and spell.english ~= 'Impact' then
            equip(sets.buff['気炎万丈の章'])
        end
        if state.Buff['震天動地の章'] then
            equip(sets.buff['震天動地の章'])
        end
        if state.Buff['虚誘掩殺の策'] and spell.element == world.weather_element then
            equip(sets.buff['虚誘掩殺の策'])
        end
    end

    if state.Buff['簡素清貧の章'] then equip(sets.buff['簡素清貧の章']) end
    if state.Buff['勤倹小心の章'] then equip(sets.buff['勤倹小心の章']) end
    if state.Buff['電光石火の章'] then equip(sets.buff['電光石火の章']) end
    if state.Buff['疾風迅雷の章'] then equip(sets.buff['疾風迅雷の章']) end
end


-- General handling of strategems in an Arts-agnostic way.
-- Format: gs c scholar <strategem>
function handle_strategems(cmdParams)
    -- cmdParams[1] == 'scholar'
    -- cmdParams[2] == strategem to use

    if not cmdParams[2] then
        add_to_chat(123,windower.to_shift_jis('Error: No strategem command given.'))
        return
    end
    local strategem = cmdParams[2]:lower()

    if strategem == 'light' then
        if buffactive['白のグリモア'] then
            send_command('input /ja '..windower.to_shift_jis("白の補遺")..' <me>')
        elseif buffactive['白の補遺'] then
            add_to_chat(122,windower.to_shift_jis('Error: 白の補遺 is already active.'))
        else
            send_command('input /ja '..windower.to_shift_jis("白のグリモア")..' <me>')
        end
    elseif strategem == 'dark' then
        if buffactive['黒のグリモア'] then
            send_command('input /ja '..windower.to_shift_jis("黒の補遺")..' <me>')
        elseif buffactive['黒の補遺'] then
            add_to_chat(122,windower.to_shift_jis('Error: 黒の補遺 is already active.'))
        else
            send_command('input /ja '..windower.to_shift_jis("黒のグリモア")..' <me>')
        end
    elseif buffactive['白のグリモア'] or buffactive['白の補遺'] then
        if strategem == 'cost' then
            send_command('input /ja '..windower.to_shift_jis("簡素清貧の章")..' <me>')
        elseif strategem == 'speed' then
            send_command('input /ja '..windower.to_shift_jis("電光石火の章")..' <me>')
        elseif strategem == 'aoe' then
            send_command('input /ja '..windower.to_shift_jis("女神降臨の章")..' <me>')
        elseif strategem == 'power' then
            send_command('input /ja '..windower.to_shift_jis("意気昂然の章")..' <me>')
        elseif strategem == 'duration' then
            send_command('input /ja '..windower.to_shift_jis("令狸執鼠の章")..' <me>')
        elseif strategem == 'accuracy' then
            send_command('input /ja '..windower.to_shift_jis("不惜身命の章")..' <me>')
        elseif strategem == 'enmity' then
            send_command('input /ja '..windower.to_shift_jis("天衣無縫の章")..' <me>')
        elseif strategem == 'skillchain' then
            add_to_chat(122,windower.to_shift_jis('Error: 白のグリモア does not have a skillchain strategem.'))
        elseif strategem == 'addendum' then
            send_command('input /ja '..windower.to_shift_jis("白の補遺")..' <me>')
        else
            add_to_chat(123,windower.to_shift_jis('Error: Unknown strategem ['..strategem..']'))
        end
    elseif buffactive['黒のグリモア']  or buffactive['黒の補遺'] then
        if strategem == 'cost' then
            send_command('input /ja '..windower.to_shift_jis("勤倹小心の章")..' <me>')
        elseif strategem == 'speed' then
            send_command('input /ja '..windower.to_shift_jis("疾風迅雷の章")..' <me>')
        elseif strategem == 'aoe' then
            send_command('input /ja '..windower.to_shift_jis("精霊光来の章")..' <me>')
        elseif strategem == 'power' then
            send_command('input /ja '..windower.to_shift_jis("気炎万丈の章")..' <me>')
        elseif strategem == 'duration' then
            add_to_chat(122,'Error: 黒のグリモア does not have a duration strategem.')
        elseif strategem == 'accuracy' then
            send_command('input /ja '..windower.to_shift_jis("一心精進の章")..' <me>')
        elseif strategem == 'enmity' then
            send_command('input /ja '..windower.to_shift_jis("無憂無風の章")..' <me>')
        elseif strategem == 'skillchain' then
            send_command('input /ja '..windower.to_shift_jis("震天動地の章")..' <me>')
        elseif strategem == 'addendum' then
            send_command('input /ja '..windower.to_shift_jis("黒の補遺")..' <me>')
        else
            add_to_chat(123,'Error: Unknown strategem ['..strategem..']')
        end
    else
        add_to_chat(123,windower.to_shift_jis('No arts has been activated yet.'))
    end
end


-- Gets the current number of available strategems based on the recast remaining
-- and the level of the sch.
function get_current_strategem_count()
    -- returns recast in seconds.
    local allRecasts = windower.ffxi.get_ability_recasts()
    local stratsRecast = allRecasts[231]

    local maxStrategems = (player.main_job_level + 10) / 20

    local fullRechargeTime = 4*60

    local currentCharges = math.floor(maxStrategems - maxStrategems * stratsRecast / fullRechargeTime)

    return currentCharges
end


-- Select default macro book on initial load or subjob change.
function select_default_macro_book()
    set_macro_page(1, 17)
end

