-------------------------------------------------------------------------------------------------------------------
-- Setup functions for this job.  Generally should not be modified.
-------------------------------------------------------------------------------------------------------------------

-- Also, you'll need the Shortcuts addon to handle the auto-targetting of the custom pact commands.

--[[
    Custom commands:
    
    gs c petweather
        Automatically casts the storm appropriate for the current avatar, if possible.
    
    gs c siphon
        Automatically run the process to: dismiss the current avatar; cast appropriate
        weather; summon the appropriate spirit; Elemental Siphon; release the spirit;
        and re-summon the avatar.
        
        Will not cast weather you do not have access to.
        Will not re-summon the avatar if one was not out in the first place.
        Will not release the spirit if it was out before the command was issued.
        
    gs c pact [PactType]
        Attempts to use the indicated pact type for the current avatar.
        PactType can be one of:
            cure
            curaga
            buffOffense
            buffDefense
            buffSpecial
            debuff1
            debuff2
            sleep
            nuke2
            nuke4
            bp70
            bp75 (merits and lvl 75-80 pacts)
            astralflow

--]]


-- Initialization function for this job file.
function get_sets()
    mote_include_version = 2
    
    -- Load and initialize the include file.
    include('Mote-Include.lua')
	set_language('japanese')
end

-- Setup vars that are user-independent.  state.Buff vars initialized here will automatically be tracked.
function job_setup()
    state.Buff["神獣の加護"] = buffactive["神獣の加護"] or false
    state.Buff["アストラルパッセージ"] = buffactive["アストラルパッセージ"] or false

spirits = S{"光精霊召喚", "闇精霊召喚", "火精霊召喚", "土精霊召喚", "水精霊召喚", "風精霊召喚", "氷精霊召喚", "雷精霊召喚"}
avatars = S{"カーバンクル","フェンリル","ディアボロス","イフリート","タイタン","リヴァイアサン","ガルーダ","シヴァ","ラムウ","ケット・シー"}

    magicalRagePacts = S{
        'インフェルノ','アースフューリー','タイダルウェイブ','エリアルブラスト','ダイヤモンドダスト','ジャッジボルト','シアリングライト','ハウリングムーン','ルイナスオーメン',
        'ファイアII','ストーンII','ウォータII','エアロII','ブリザドII','サンダーII',
        'ファイアIV','ストーンIV','ウォータIV','エアロIV','ブリザドIV','サンダーIV',
        'サンダースパーク','バーニングストライク','プチメテオ','ネザーブラスト','フレイムクラッシュ',
        'メテオストライク','ヘヴンリーストライク','ウインドブレード','ジオクラッシュ','グランドフォール','サンダーストーム',
        'ホーリーミスト','ルナーベイ','ナイトテラー','レベル？ホーリー',
		'コンフラグストライク'}

    pacts = {}
    pacts.cure = {['カーバンクル']='ルビーの癒し'}
    pacts.curaga = {['カーバンクル']='ルビーの癒しII', ['ガルーダ']='風の囁き', ['リヴァイアサン']='湧水'}
    pacts.buffoffense = {['カーバンクル']='ルビーの煌き', ['イフリート']='紅蓮の咆哮', ['ガルーダ']='ヘイスガ', ['ラムウ']='雷鼓',
        ['フェンリル']='上弦の唸り'}
    pacts.buffdefense = {['カーバンクル']='ルビーの輝き', ['シヴァ']='凍てつく鎧', ['ガルーダ']='真空の鎧', ['タイタン']='大地の守り',
        ['ラムウ']='雷電の鎧', ['フェンリル']='下弦の咆哮', ['ディアボロス']='ノクトシールド', ['ケット・シー']='リレイズII'}
    pacts.buffspecial = {['イフリート']='灼熱の咆哮', ['ガルーダ']='真空の具足', ['タイタン']='大地の鎧', ['ディアボロス']='ドリームシュラウド',
        ['カーバンクル']='ルビーの安らぎ', ['フェンリル']='ヘヴンズハウル', ['ケット・シー']='レイズII'}
    pacts.debuff1 = {['シヴァ']='ダイヤモンドストーム', ['ラムウ']='スタンガ', ['リヴァイアサン']='タイダルロア', ['フェンリル']='ルナークライ',
        ['ディアボロス']='パボルノクターナス', ['ケット・シー']='イアリーアイ'}
    pacts.debuff2 = {['シヴァ']='スリプガ', ['リヴァイアサン']='スロウガ', ['フェンリル']='ルナーロア', ['ディアボロス']='ソムノレンス'}
    pacts.sleep = {['シヴァ']='スリプガ', ['ディアボロス']='ナイトメア', ['ケット・シー']='ミュインララバイ'}
    pacts.nuke2 = {['イフリート']='ファイアII', ['シヴァ']='ブリザドII', ['ガルーダ']='エアロII', ['タイタン']='ストーンII',
        ['ラムウ']='サンダーII', ['リヴァイアサン']='ウォータII'}
    pacts.nuke4 = {['イフリート']='ファイアIV', ['シヴァ']='ブリザドIV', ['ガルーダ']='エアロIV', ['タイタン']='ストーンIV',
        ['ラムウ']='サンダーIV', ['リヴァイアサン']='ウォータIV'}
    pacts.bp70 = {['イフリート']='フレイムクラッシュ', ['シヴァ']='ラッシュ', ['ガルーダ']='プレデタークロー', ['タイタン']='マウンテンバスター',
        ['ラムウ']='カオスストライク', ['リヴァイアサン']='スピニングダイブ', ['カーバンクル']='プチメテオ', ['フェンリル']='エクリプスバイト',
        ['ディアボロス']='ネザーブラスト',['ケット・シー']='リーガルスクラッチ'}
    pacts.bp75 = {['イフリート']='メテオストライク', ['シヴァ']='ヘヴンリーストライク', ['ガルーダ']='ウインドブレード', ['タイタン']='ジオクラッシュ',
        ['ラムウ']='サンダーストーム', ['リヴァイアサン']='グランドフォール', ['カーバンクル']='ホーリーミスト', ['フェンリル']='ルナーベイ',
        ['ディアボロス']='ナイトテラー', ['ケット・シー']='レベル？ホーリー'}
    pacts.astralflow = {['イフリート']='インフェルノ', ['シヴァ']='ダイヤモンドダスト', ['ガルーダ']='エリアルブラスト', ['タイタン']='アースフューリー',
        ['ラムウ']='ジャッジボルト', ['リヴァイアサン']='タイダルウェイブ', ['カーバンクル']='シアリングライト', ['フェンリル']='ハウリングムーン',
        ['ディアボロス']='ルイナスオーメン', ['ケット・シー']="アルタナフェーバー"}

    -- Wards table for creating custom timers   
    wards = {}
    -- Base duration for ward pacts.
    wards.durations = {
        ['紅蓮の咆哮'] = 60, ['大地の鎧'] = 60, ['灼熱の咆哮'] = 60, ['ヘヴンズハウル'] = 60,
        ['雷鼓'] = 120, ['真空の具足'] = 120,
        ['ルビーの輝き'] = 180, ['凍てつく鎧'] = 180, ['雷電の鎧'] = 180, ['上弦の唸り'] = 180,
        ['ルビーの煌き'] = 180, ['ヘイスガ'] = 180, ['ノクトシールド'] = 180, ['下弦の咆哮'] = 180,
        ['ドリームシュラウド'] = 180,
        ['リレイズII'] = 3600
    }
    -- Icons to use when creating the custom timer.
    wards.icons = {
        ['大地の鎧']   = 'spells/00299.png', -- 00299 for タイタン
        ['ルビーの輝き']    = 'spells/00043.png', -- 00043 for Protect
        ['ドリームシュラウド']    = 'spells/00304.png', -- 00304 for ディアボロス
        ['ノクトシールド']     = 'spells/00106.png', -- 00106 for Phalanx
        ['灼熱の咆哮']    = 'spells/00298.png', -- 00298 for イフリート
        ['ヘイスガ']         = 'spells/00358.png', -- 00358 for ヘイスガ
        ['雷鼓'] = 'spells/00104.png', -- 00358 for Enthunder
        ['凍てつく鎧']     = 'spells/00250.png', -- 00250 for Ice Spikes
        ['雷電の鎧'] = 'spells/00251.png', -- 00251 for Shock Spikes
        ['リレイズII']      = 'spells/00135.png', -- 00135 for Reraise
        ['真空の具足']      = 'abilities/00074.png', -- 
    }
    -- Flags for code to get around the issue of slow skill updates.
    wards.flag = false
    wards.spell = ''
    
end

-------------------------------------------------------------------------------------------------------------------
-- User setup functions for this job.  Recommend that these be overridden in a sidecar file.
-------------------------------------------------------------------------------------------------------------------

-- Setup vars that are user-dependent.  Can override this function in a sidecar file.
function user_setup()
    state.OffenseMode:options('Normal', 'Acc')
    state.CastingMode:options('Normal', 'Resistant')
    state.IdleMode:options('Normal', 'PDT')

    gear.perp_staff = {name=""}
    
    --select_default_macro_book()
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for standard casting events.
-------------------------------------------------------------------------------------------------------------------

-- Set eventArgs.handled to true if we don't want any automatic gear equipping to be done.
-- Set eventArgs.useMidcastGear to true if we want midcast gear equipped on precast.
function job_precast(spell, action, spellMap, eventArgs)
    if state.Buff['アストラルパッセージ'] and pet_midaction() then
        eventArgs.handled = true
    end
end

function job_midcast(spell, action, spellMap, eventArgs)
    if state.Buff['アストラルパッセージ'] and pet_midaction() then
        eventArgs.handled = true
    end
end

-- Runs when pet completes an action.
function job_pet_aftercast(spell, action, spellMap, eventArgs)
    if not spell.interrupted and spell.type == 'BloodPactWard' and spellMap ~= 'DebuffBloodPactWard' then
        wards.flag = true
        wards.spell = spell.name
        send_command('wait 4; gs c reset_ward_flag')
    end
end

-------------------------------------------------------------------------------------------------------------------
-- Job-specific hooks for non-casting events.
-------------------------------------------------------------------------------------------------------------------

-- Called when a player gains or loses a buff.
-- buff == buff gained or lost
-- gain == true if the buff was gained, false if it was lost.
function job_buff_change(buff, gain)
    if state.Buff[buff] ~= nil then
        handle_equipping_gear(player.status)
    elseif storms:contains(buff) then
        handle_equipping_gear(player.status)
    end
end


-- Called when the player's pet's status changes.
-- This is also called after pet_change after a pet is released.  Check for pet validity.
function job_pet_status_change(newStatus, oldStatus, eventArgs)
    if pet.isvalid and not midaction() and not pet_midaction() and (newStatus == 'Engaged' or oldStatus == 'Engaged') then
        handle_equipping_gear(player.status, newStatus)
    end
end


-- Called when a player gains or loses a pet.
-- pet == pet structure
-- gain == true if the pet was gained, false if it was lost.
function job_pet_change(petparam, gain)
    classes.CustomIdleGroups:clear()
    if gain then
        if avatars:contains(pet.name) then
            classes.CustomIdleGroups:append('Avatar')
        elseif spirits:contains(pet.name) then
            classes.CustomIdleGroups:append('Spirit')
        end
    else
        select_default_macro_book('reset')
    end
end

-------------------------------------------------------------------------------------------------------------------
-- User code that supplements standard library decisions.
-------------------------------------------------------------------------------------------------------------------

-- Custom spell mapping.
function job_get_spell_map(spell)
    if spell.type == 'BloodPactRage' then
        if magicalRagePacts:contains(spell.english) then
            return 'MagicalBloodPactRage'
        else
            return 'PhysicalBloodPactRage'
        end
    elseif spell.type == 'BloodPactWard' and spell.target.type == 'MONSTER' then
        return 'DebuffBloodPactWard'
    end
end

-- Modify the default idle set after it was constructed.
function customize_idle_set(idleSet)
    if pet.isvalid then
        if pet.element == world.day_element then
            idleSet = set_combine(idleSet, sets.perp.Day)
        end
        if pet.element == world.weather_element then
            idleSet = set_combine(idleSet, sets.perp.Weather)
        end
        if sets.perp[pet.name] then
            idleSet = set_combine(idleSet, sets.perp[pet.name])
        end
        gear.perp_staff.name = elements.perpetuance_staff_of[pet.element]
        if gear.perp_staff.name and (player.inventory[gear.perp_staff.name] or player.wardrobe[gear.perp_staff.name]) then
            idleSet = set_combine(idleSet, sets.perp.staff_and_grip)
        end
        if state.Buff["神獣の加護"] and avatars:contains(pet.name) then
            idleSet = set_combine(idleSet, sets.idle.Avatar.Favor)
        end
        if pet.status == 'Engaged' then
            idleSet = set_combine(idleSet, sets.idle.Avatar.Melee)
        end
    end
    
    if player.mpp < 51 then
        idleSet = set_combine(idleSet, sets.latent_refresh)
    end
    
    return idleSet
end

-- Called by the 'update' self-command, for common needs.
-- Set eventArgs.handled to true if we don't want automatic equipping of gear.
function job_update(cmdParams, eventArgs)
    classes.CustomIdleGroups:clear()
    if pet.isvalid then
        if avatars:contains(pet.name) then
            classes.CustomIdleGroups:append('Avatar')
        elseif spirits:contains(pet.name) then
            classes.CustomIdleGroups:append('Spirit')
        end
    end
end

-- Set eventArgs.handled to true if we don't want the automatic display to be run.
function display_current_job_state(eventArgs)

end


-------------------------------------------------------------------------------------------------------------------
-- User self-commands.
-------------------------------------------------------------------------------------------------------------------

-- Called for custom player commands.
function job_self_command(cmdParams, eventArgs)
    if cmdParams[1]:lower() == 'petweather' then
        handle_petweather()
        eventArgs.handled = true
    elseif cmdParams[1]:lower() == 'siphon' then
        handle_siphoning()
        eventArgs.handled = true
    elseif cmdParams[1]:lower() == 'pact' then
        handle_pacts(cmdParams)
        eventArgs.handled = true
    elseif cmdParams[1] == 'reset_ward_flag' then
        wards.flag = false
        wards.spell = ''
        eventArgs.handled = true
    end
end


-------------------------------------------------------------------------------------------------------------------
-- Utility functions specific to this job.
-------------------------------------------------------------------------------------------------------------------

-- Cast the appopriate storm for the currently summoned avatar, if possible.
function handle_petweather()
    if player.sub_job ~= '学' then
        add_to_chat(122, "You can not cast storm spells")
        return
    end
        
    if not pet.isvalid then
        add_to_chat(122, "You do not have an active avatar.")
        return
    end
       
    local storm = elements.storm_of[element]
    
    if storm then
        send_command('@input /ma "'..elements.storm_of[element]..'" <me>')
    else
        add_to_chat(123, 'Error: Unknown element ('..tostring(element)..')')
    end
end


-- Custom uber-handling of Elemental Siphon
function handle_siphoning()
    if areas.Cities:contains(world.area) then
        add_to_chat(122, 'Cannot use Elemental Siphon in a city area.')
        return
    end

    local siphonElement
    local stormElementToUse
    local releasedAvatar
    local dontRelease
    
    -- If we already have a spirit out, just use that.
    if pet.isvalid and spirits:contains(pet.name) then
        siphonElement = pet.element
        dontRelease = true
        -- If current weather doesn't match the spirit, but the spirit matches the day, try to cast the storm.
        if player.sub_job == '学' and pet.element == world.day_element and pet.element ~= world.weather_element then
			stormElementToUse = pet.element
        end
    -- If we're subbing /sch, there are some conditions where we want to make sure specific weather is up.
    -- If current (single) weather is opposed by the current day, we want to change the weather to match
    -- the current day, if possible.
    elseif player.sub_job == '学' and world.weather_element ~= 'なし' then
        -- We can override single-intensity weather; leave double weather alone, since even if
        -- it's partially countered by the day, it's not worth changing.
        if get_weather_intensity() == 1 then
            -- If current weather is weak to the current day, it cancels the benefits for
            -- siphon.  Change it to the day's weather if possible (+0 to +20%), or any non-weak
            -- weather if not.
            -- If the current weather matches the current avatar's element (being used to reduce
            -- perpetuation), don't change it; just accept the penalty on Siphon.
            if world.weather_element == elements.weak_to[world.day_element] and
                (not pet.isvalid or world.weather_element ~= pet.element) then
                -- We can't cast lightning/dark/light weather, so use a neutral element
                    stormElementToUse = world.day_element
            end
        end
    end
    
    -- If we decided to use a storm, set that as the spirit element to cast.
    if stormElementToUse then
        siphonElement = stormElementToUse
    elseif world.weather_element ~= 'なし' and (get_weather_intensity() == 2 or world.weather_element ~= elements.weak_to[world.day_element]) then
        siphonElement = world.weather_element
    else
        siphonElement = world.day_element
    end
    
    local command = ''
    local releaseWait = 0
    
    if pet.isvalid and avatars:contains(pet.name) then
        command = command..'input /pet '..windower.to_shift_jis("神獣の帰還")..' <me>;wait 1.1;'
        releasedAvatar = pet.name
        releaseWait = 10
    end
    
    if stormElementToUse then
        command = command..'input /ma '..windower.to_shift_jis(elements.storm_of[stormElementToUse])..' <me>;wait 4;'
        releaseWait = releaseWait - 4
    end
    
    if not (pet.isvalid and spirits:contains(pet.name)) then
        command = command..'input /ma '..windower.to_shift_jis(elements.spirit_of[siphonElement])..' <me>;wait 4;'
        releaseWait = releaseWait - 4
    end
    
    command = command..'input /ja '..windower.to_shift_jis("エレメントサイフォン")..' <me>;'
    releaseWait = releaseWait - 1
    releaseWait = releaseWait + 0.1
    
    if not dontRelease then
        if releaseWait > 0 then
            command = command..'wait '..windower.to_shift_jis(tostring(releaseWait))..';'
        else
            command = command..'wait 1.1;'
        end
        
        command = command..'input /pet '..windower.to_shift_jis("神獣の帰還")..' <me>;'
    end
    
    if releasedAvatar then
        command = command..'wait 1.1;input /ma '..windower.to_shift_jis(releasedAvatar)..' <me>'
    end
    
    send_command(command)
end


-- Handles executing blood pacts in a generic, avatar-agnostic way.
-- cmdParams is the split of the self-command.
-- gs c [pact] [pacttype]
function handle_pacts(cmdParams)
    if areas.Cities:contains(world.area) then
        add_to_chat(122, 'You cannot use pacts in town.')
        return
    end

    if not pet.isvalid then
        add_to_chat(122,'No avatar currently available. Returning to default macro set.')
        --select_default_macro_book('reset')
        return
    end

    if spirits:contains(pet.name) then
        add_to_chat(122,'Cannot use pacts with spirits.')
        return
    end

    if not cmdParams[2] then
        add_to_chat(123,'No pact type given.')
        return
    end
    
    local pact = cmdParams[2]:lower()
    
    if not pacts[pact] then
        add_to_chat(123,'Unknown pact type: '..tostring(pact))
        return
    end
    
    if pacts[pact][pet.name] then
        if pact == 'astralflow' and not buffactive['アストラルフロウ'] then
            add_to_chat(122,'Cannot use Astral Flow pacts at this time.')
            return
        end
        
        -- Leave out target; let Shortcuts auto-determine it.
        send_command('@input /pet '..windower.to_shift_jis(pacts[pact][pet.name]))
    else
        add_to_chat(122,pet.name..' does not have a pact of type ['..pact..'].')
    end
end


-- Event handler for updates to player skill, since we can't rely on skill being
-- correct at pet_aftercast for the creation of custom timers.
windower.raw_register_event('incoming chunk',
    function (id)
        if id == 0x62 then
            if wards.flag then
                create_pact_timer(wards.spell)
                wards.flag = false
                wards.spell = ''
            end
        end
    end)

-- Function to create custom timers using the Timers addon.  Calculates ward duration
-- based on player skill and base pact duration (defined in job_setup).
function create_pact_timer(spell_name)
    -- Create custom timers for ward pacts.
    if wards.durations[spell_name] then
        local ward_duration = wards.durations[spell_name]
        if ward_duration < 181 then
            local skill = player.skills.summoning_magic
            if skill > 300 then
                skill = skill - 300
                if skill > 200 then skill = 200 end
                ward_duration = ward_duration + skill
            end
        end
        
        local timer_cmd = 'timers c "'..windower.to_shift_jis(spell_name)..'" '..windower.to_shift_jis(tostring(ward_duration))' down'
        
        if wards.icons[spell_name] then
            timer_cmd = timer_cmd..' '..wards.icons[spell_name]
        end

        send_command(timer_cmd)
    end
end


-- Select default macro book on initial load or subjob change.
function select_default_macro_book(reset)
    if reset == 'reset' then
        -- lost pet, or tried to use pact when pet is gone
    end
    
    -- Default macro set/book
    set_macro_page(4, 16)
end
