-------------------------------------------------------------------------------------------------------------------
-- Mappings, lists and sets to describe game relationships that aren't easily determinable otherwise.
-------------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------------
-- Elemental mappings for element relationships and certain types of spells and gear.
-------------------------------------------------------------------------------------------------------------------

-- Basic elements
elements = {}

elements.list = S{'光','闇','火','氷','風','土','雷','水'}
elements.weak_to = {['光']='闇', ['闇']='光', ['火']='氷', ['氷']='風', ['風']='土', ['土']='雷',['雷']='水', ['水']='火'}
elements.strong_to = {['光']='闇', ['闇']='光', ['火']='水', ['氷']='火', ['風']='氷', ['土']='風',['雷']='土', ['水']='雷'}

storms = S{"極光の陣","妖霧の陣","熱波の陣","砂塵の陣","豪雨の陣","烈風の陣","吹雪の陣","疾雷の陣","極光の陣II","妖霧の陣II","熱波の陣II","砂塵の陣II","豪雨の陣II","烈風の陣II","吹雪の陣II","疾雷の陣II"}
helix = S{'土門の計','水門の計','風門の計','火門の計','氷門の計','雷門の計','闇門の計','光門の計','土門の計II','水門の計II','風門の計II','火門の計II','氷門の計II','雷門の計II','闇門の計II','光門の計II',}
elements.storm_of = {['光']="極光の陣", ['闇']="妖霧の陣", ['火']="熱波の陣", ['土']="砂塵の陣",['水']="豪雨の陣", ['風']="烈風の陣", ['氷']="吹雪の陣", ['雷']="疾雷の陣"}

spirits = S{"光精霊召喚", "闇精霊召喚", "火精霊召喚", "土精霊召喚", "水精霊召喚", "風精霊召喚", "氷精霊召喚", "雷精霊召喚"}
avatars = S{"カーバンクル","フェンリル","ディアボロス","イフリート","タイタン","リヴァイアサン","ガルーダ","シヴァ","ラムウ","ケット・シー","セイレーン"}
elements.spirit_of = {['光']="光精霊召喚", ['闇']="闇精霊召喚", ['火']="火精霊召喚", ['土']="土精霊召喚",['水']="水精霊召喚", ['風']="風精霊召喚", ['氷']="氷精霊召喚", ['雷']="雷精霊召喚"}

runes = S{'ルックス', 'テネブレイ', 'イグニス', 'ゲールス', 'フラブラ', 'テッルス', 'スルポール', 'ウンダ'}
elements.rune_of = {['光']='ルックス', ['闇']='テネブレイ', ['火']='イグニス', ['氷']='ゲールス', ['風']='フラブラ',['土']='テッルス', ['雷']='スルポール', ['水']='ウンダ'}

elements.obi_of = {['光']='八輪の帯', ['闇']='八輪の帯', ['火']='八輪の帯', ['氷']='八輪の帯', ['風']='八輪の帯', ['土']='八輪の帯', ['雷']='八輪の帯', ['水']='八輪の帯'}
elements.gorget_of = {['光']='フォシャゴルゲット', ['闇']='フォシャゴルゲット', ['火']='フォシャゴルゲット', ['氷']='フォシャゴルゲット', ['風']='フォシャゴルゲット', ['土']='フォシャゴルゲット', ['雷']='フォシャゴルゲット', ['水']='フォシャゴルゲット'}
elements.belt_of = {['光']='フォシャベルト', ['闇']='フォシャベルト', ['火']='フォシャベルト', ['氷']='フォシャベルト', ['風']='フォシャベルト', ['土']='フォシャベルト', ['雷']='フォシャベルト', ['水']='フォシャベルト'}
elements.fastcast_staff_of = {['光']='アーカI', ['闇']='キサエタI', ['火']='アターI', ['氷']='ボルカシャI', ['風']='バユバタI', ['土']='ビシュラバI', ['雷']='アパマジャI', ['水']='ハオーマI'}
elements.recast_staff_of = {['光']='アーカII', ['闇']='キサエタII', ['火']='アターII', ['氷']='ボルカシャII', ['風']='バユバタII', ['土']='ビシュラバII', ['雷']='アパマジャ II', ['水']='ハオーマII'}
elements.perpetuance_staff_of = {['光']='アーカIII', ['闇']='キサエタIII', ['火']='アターIII', ['氷']='ボルカシャIII', ['風']='バユバタIII', ['土']='ビシュラバIII', ['雷']='アパマジャIII', ['水']='ハオーマIII'}


-- Elements for skillchain names
skillchain_elements = {}
skillchain_elements.Light = S{'光','火','風','雷'}
skillchain_elements.Darkness = S{'闇','氷','土','水'}
skillchain_elements.Fusion = S{'光','火'}
skillchain_elements.Fragmentation = S{'風','雷'}
skillchain_elements.Distortion = S{'氷','水'}
skillchain_elements.Gravitation = S{'闇','土'}
skillchain_elements.Transfixion = S{'光'}
skillchain_elements.Compression = S{'闇'}
skillchain_elements.Liquification = S{'火'}
skillchain_elements.Induration = S{'氷'}
skillchain_elements.Detonation = S{'風'}
skillchain_elements.Scission = S{'土'}
skillchain_elements.Impaction = S{'雷'}
skillchain_elements.Reverberation = S{'水'}


-------------------------------------------------------------------------------------------------------------------
-- Mappings for weaponskills
-------------------------------------------------------------------------------------------------------------------

-- REM weapons and their corresponding weaponskills
data = {}
data.weaponskills = {}
data.weaponskills.relic = {
    ["スファライ"] = "ファイナルヘヴン",
    ["マンダウ"] = "マーシーシトローク",
    ["エクスカリバー"] = "ナイツオブラウンド",
    ["ラグナロク"] = "スカージ",
    ["ガトラー"] = "オンスロート",
    ["ラブビューラ"] = "メタトロントーメント",
    ["アポカリプス"] = "カタストロフィ",
    ["グングニル"] = "ゲイルスコグル",
    ["鬼哭"] = "生者必滅",
    ["天の村雲"] = "零之太刀:回天",
    ["ミョルニル"] = "ランドグリース",
    ["クラウストルム"] = "タルタロスゲート",
    ["アナイアレーター"] = "カラナック",
    ["与一の弓"] = "南無八幡"}
data.weaponskills.mythic = {
    ["コンカラー"] = "キングジャスティス",
    ["グランツファウスト"] = "アスケーテンツォルン",
    ["ヤグルシュ"] = "ミスティックブーン",
    ["レーヴァテイン"] = "ヴィゾフニル",
    ["ミュルグレス"] = "ロズレーファタール",
    ["ヴァジュラ"] = "マンダリクスタップ",
    ["ブルトガング"] = "ロイエ",
    ["リベレーター"] = "インサージェンシー",
    ["アイムール"] = "プライマルレンド",
    ["カルンウェナン"] = "モーダントライム",
    ["ガストラフェテス"] = "トゥルーフライト",
    ["小鴉丸"] = "十之太刀:乱鴉",
    ["凪"] = "カムハブリ",
    ["竜の髭"] = "雲蒸竜変",
    ["ニルヴァーナ"] = "ガーランドオブブリス",
    ["ティソーナ"] = "エクスピアシオン",
    ["デスペナルティ"] = "レデンサリュート",
    ["乾坤圏"] = "連環六合圏",
    ["テルプシコラー"] = "ピリッククレオス",
    ["トゥプシマティ"] = "オムニシエンス",
    ["イドリス"] = "エクズデーション",
    ["エピオラトリー"] = "デミディエーション"}
data.weaponskills.empyrean = {
    ["ウルスラグナ"] = "ビクトリースマイト",
    ["トゥワシュトラ"] = "ルドラストーム",
    ["アルマス"] = "シャンデュシニュ",
    ["カラドボルグ"] = "トアクリーバー",
    ["ファルシャ"] = "クラウドスプリッタ",
    ["ウコンバサラ"] = "ウッコフューリー",
    ["リデンプション"] = "クワイタス",
    ["ロンゴミアント"] = "カムラン",
    ["神無"] = "秘",
    ["正宗"] = "祖之太刀:不動",
    ["ガンバンテイン"] = "ダガン",
    ["フヴェルゲルミン"] = "ミルキル",
    ["ガーンデーヴァ"] = "ジシュヌの光輝",
    ["アルマゲドン"] = "ワイルドファイア"}

-- Weaponskills that can be used at range.
data.weaponskills.ranged = S{"フレイミングアロー", "ピアシングアロー", "ダリングアロー", "サイドワインダー", "アーチングアロー",
    "エンピリアルアロー", "リフルジェントアロー", "エイペクスアロー", "南無八幡", "ジシュヌの光輝",
    "ホットショット", "スプリットショット", "スナイパーショット", "スラッグショット", "ヘヴィショット", "デトネーター", "ラストスタンド",
    "カラナック", "トゥルーフライト", "レデンサリュート", "ワイルドファイア",
    "ミルキル", "ダガン"}

ranged_weaponskills = data.weaponskills.ranged

-------------------------------------------------------------------------------------------------------------------
-- Spell mappings allow defining a general category or description that each of sets of related
-- spells all fall under.
-------------------------------------------------------------------------------------------------------------------

spell_maps = {
	['ケアル']='Cure',['ケアルII']='Cure',['ケアルIII']='Cure',['ケアルIV']='Cure',['ケアルV']='Cure',['ケアルVI']='Cure',['フルケア']='Cure',
	['ケアルラ']='Curaga',['ケアルラII']='Curaga',['ケアルラIII']='Curaga',['ケアルガ']='Curaga',['ケアルガII']='Curaga',['ケアルガIII']='Curaga',['ケアルガIV']='Curaga',['ケアルガV']='Curaga',
    -- Status Removal doesn't include Esuna or Sacrifice, since they work differently than the rest
	['ポイゾナ']='StatusRemoval',['パラナ']='StatusRemoval',['サイレナ']='StatusRemoval',['ブライナ']='StatusRemoval',
	['ストナ']='StatusRemoval',['ウィルナ']='StatusRemoval',['イレース']='StatusRemoval',
	['カーズナ']='Cursna',
	['バファイ']='BarElement',['バストン']='BarElement',['バウォタ']='BarElement',['バエアロ']='BarElement',['バブリザ']='BarElement',['バサンダ']='BarElement',
	['バファイラ']='BarElement',['バストンラ']='BarElement',['バウォタラ']='BarElement',['バエアロラ']='BarElement',['バブリザラ']='BarElement',['バサンダラ']='BarElement',
	['バスリプル']='BarStatus',['バポイズン']='BarStatus',['バブライン']='BarStatus',['バパライズ']='BarStatus',['バサイレス']='BarStatus',['バウィルス']='BarStatus',['バブレイク']='BarStatus',['バアムネジア']='BarStatus',
	['バスリプラ']='BarStatus',['バポイゾラ']='BarStatus',['バブライラ']='BarStatus',['バパライラ']='BarStatus',['バサイレラ']='BarStatus',['バウィルラ']='BarStatus',['バブレクラ']='BarStatus',['バアムネジラ']='BarStatus',
	['アディバイト']='BoostStat',['アディマイン']='BoostStat',['アディカリス']='BoostStat',['アディアジル']='BoostStat',['アディスト']='BoostStat',['アディイン']='BoostStat',['アディデック']='BoostStat',
	['ゲインバイト']='BoostStat',['ゲインマイン']='BoostStat',['ゲインカリス']='BoostStat',['ゲインアジル']='BoostStat',['ゲインスト']='BoostStat',['ゲインイン']='BoostStat',['ゲインデック']='BoostStat',
	['レイズ']='Raise',['レイズII']='Raise',['レイズIII']='Raise',['アレイズ']='Raise',
	['リレイズ']='Reraise',['リレイズII']='Reraise',['リレイズIII']='Reraise',['リレイズIV']='Reraise',
	['ディア']='Dia',['ディアII']='Dia',['ディアIII']='Dia',['ディアガ']='Dia',['ディアガII']='Dia',['ディアガIII']='Dia',
	['バイオ']='Bio',['バイオII']='Bio',['バイオIII']='Bio',
	['ディスペル']='Dispel',['ディスペガ']='Dispel',
	['プロテス']='Protect',['プロテスII']='Protect',['プロテスIII']='Protect',['プロテスIV']='Protect',['プロテスV']='Protect',
	['シェル']='Protect',['シェルII']='Protect',['シェルIII']='Protect',['シェルIV']='Protect',['シェルV']='Protect',
	['プロテア']='Protect',['プロテアII']='Protect',['プロテアIII']='Protect',['プロテアIV']='Protect',['プロテアV']='Protect',
	['シェルラ']='Protect',['シェルラII']='Protect',['シェルラIII']='Protect',['シェルラIV']='Protect',['シェルラV']='Protect',
	['リジェネ']='Regen',['リジェネII']='Regen',['リジェネIII']='Regen',['リジェネIV']='Regen',['リジェネV']='Regen',
	['リフレシュ']='Refresh',['リフレシュII']='Refresh',['リフレシュIII']='Refresh',
	['テレポホラ']='Teleport',['テレポデム']='Teleport',['テレポメア']='Teleport',['テレポルテ']='Teleport',['テレポヨト']='Teleport',
	['テレポヴァズ']='Teleport',['リコールパシュ']='Teleport',['リコールメリファ']='Teleport',['リコールジャグ']='Teleport',
	['リトレース']='Teleport',['トラクタ']='Teleport',['エスケプ']='Teleport',['デジョン']='Teleport',['デジョンII']='Teleport',
	['ストライ']='Temper',['ストライII']='Temper',['ヘイスト']='Haste',['ヘイストII']='Haste',
	['ファランクス']='Phalanx',['ファランクスII']='Phalanx',
	['ブレイズスパイク']='Spikes',['アイススパイク']='Spikes',['ショックスパイク']='Spikes',
	['エンサンダー']='EnSpell',['エンストーン']='EnSpell',['エンエアロ']='EnSpell',['エンブリザド']='EnSpell',['エンファイア']='EnSpell',['エンウォータ']='EnSpell',
	['エンサンダーII']='EnSpell',['エンストーンII']='EnSpell',['エンエアロII']='EnSpell',['エンブリザドII']='EnSpell',['エンファイアII']='EnSpell',['エンウォータII']='EnSpell',
	['エンライト']='Enlight',['エンライトII']='Enlight',['エンダーク']='Endark',['エンダークII']='Endark',
	['インパクト']='Impact',['ストンスキン'] = 'Stoneskin',
	['バインド']='Bind',['スリプル']='Sleep',['スリプルII']='Sleep',['スリプガ']='Sleep',['スリプガII']='Sleep',['ブレイク']='Break',['ブレクガ']='Break',['サイレス']='Silence',	
	['スロウ']='EnfeeblingMagiceffect',['スロウII']='EnfeeblingMagiceffect',['パライズ']='EnfeeblingMagiceffect',['パライズII']='EnfeeblingMagiceffect',
	['ブライン']='EnfeeblingMagiceffect',['ブラインII']='EnfeeblingMagiceffect',['ポイズン']='EnfeeblingMagiceffect',['ポイズンII']='EnfeeblingMagiceffect',
	['グラビデ']='EnfeeblingMagiceffect',['グラビデII']='EnfeeblingMagiceffect',['ディストラ']='EnfeeblingMagiceffect',['ディストラII']='EnfeeblingMagiceffect',['ディストラIII']='EnfeeblingMagiceffect',
	['フラズル']='EnfeeblingMagiceffect',['フラズルII']='EnfeeblingMagiceffect',['フラズルIII']='EnfeeblingMagiceffect',['アドル']='EnfeeblingMagiceffect',['アドルII']='EnfeeblingMagiceffect',
	['バニシュ']='Banish',['バニシュII']='Banish',['バニシュIII']='Banish',['バニシュガ']='Banish',['バニシュガII']='Banish',
	['フラッシュ']='Flash',
	['ホーリー']='Holy',['ホーリーII']='Holy',['ドレイン']='Drain',['ドレインII']='Drain',['ドレインIII']='Drain',['アスピル']='Drain',['アスピルII']='Drain',['アスピルIII']='Drain',
	['アブゾースト']='Absorb',['アブゾデック']='Absorb',['アブゾバイト']='Absorb',['アブゾアジル']='Absorb',['アブゾイン']='Absorb',['アブゾマイン']='Absorb',['アブゾカリス']='Absorb',
	['アブゾアキュル']='Absorb',['アブゾタック']='Absorb',['アブゾアトリ']='Absorb',
	['バーン']='ElementalEnfeeble',['フロスト']='ElementalEnfeeble',['チョーク']='ElementalEnfeeble',['ラスプ']='ElementalEnfeeble',['ショック']='ElementalEnfeeble',['ドラウン']='ElementalEnfeeble',
	['土門の計']='Helix',['水門の計']='Helix',['火門の計']='Helix',['氷門の計']='Helix',['雷門の計']='Helix',['風門の計']='Helix',['光門の計']='Helix',['闇門の計']='Helix',
	['土門の計II']='Helix',['水門の計II']='Helix',['火門の計II']='Helix',['氷門の計II']='Helix',['雷門の計II']='Helix',['風門の計II']='Helix',['光門の計II']='Helix',['闇門の計II']='Helix',
	['砂塵の陣']='Storm',['豪雨の陣']='Storm',['熱波の陣']='Storm',['吹雪の陣']='Storm',['疾雷の陣']='Storm',['烈風の陣']='Storm',['極光の陣']='Storm',['妖霧の陣']='Storm',
	['砂塵の陣II']='Storm',['豪雨の陣II']='Storm',['熱波の陣II']='Storm',['吹雪の陣II']='Storm',['疾雷の陣II']='Storm',['烈風の陣II']='Storm',['極光の陣II']='Storm',['妖霧の陣II']='Storm',
	
	['猛者のメヌエット']='Minuet',['猛者のメヌエットII']='Minuet',['猛者のメヌエットIII']='Minuet',['猛者のメヌエットIV']='Minuet',['猛者のメヌエットV']='Minuet',
	["重装騎兵のミンネ"]='Minne',["重装騎兵のミンネII"]='Minne',["重装騎兵のミンネIII"]='Minne',["重装騎兵のミンネIV"]='Minne',["重装騎兵のミンネV"]='Minne',
	['無敵の進撃マーチ']='March',['栄光の凱旋マーチ']='March',['栄典の戴冠マーチ']='March',
	['剣闘士のマドリガル']='Madrigal',['剣豪のマドリガル']='Madrigal',
	["狩人のプレリュード"]='Prelude',["弓師のプレリュード"]='Prelude',
	['闘羊士のマンボ']='Mambo',['闘龍士のマンボ']='Mambo',
	['ラプトルのマズルカ']='Mazurka',['チョコボのマズルカ']='Mazurka',
	['剛力のエチュード']='Etude',['器用のエチュード']='Etude',['元気のエチュード']='Etude',['機敏のエチュード']='Etude',['知恵のエチュード']='Etude',['精神のエチュード']='Etude',['魅了のエチュード']='Etude',
	['怪力のエチュード']='Etude',['妙技のエチュード']='Etude',['活力のエチュード']='Etude',['俊敏のエチュード']='Etude',['英知のエチュード']='Etude',['理力のエチュード']='Etude',['魅惑のエチュード']='Etude',
	["魔道士のバラード"]='Ballad',["魔道士のバラードII"]='Ballad',["魔道士のバラードIII"]='Ballad',
	["戦士達のピーアン"]='Paeon',["戦士達のピーアンII"]='Paeon',["戦士達のピーアンIII"]='Paeon',["戦士達のピーアンIV"]='Paeon',["戦士達のピーアンV"]='Paeon',["戦士達のピーアンVI"]='Paeon',
	['耐火カロル第一楽章']='Carol',['耐寒カロル第一楽章']='Carol',['耐風カロル第一楽章']='Carol',['耐震カロル第一楽章']='Carol',['耐電カロル第一楽章']='Carol',['耐波カロル第一楽章']='Carol',['耐光カロル第一楽章']='Carol',['耐闇カロル第一楽章']='Carol',
	['耐火カロル第二楽章']='Carol',['耐寒カロル第二楽章']='Carol',['耐風カロル第二楽章']='Carol',['耐震カロル第二楽章']='Carol',['耐電カロル第二楽章']='Carol',['耐波カロル第二楽章']='Carol',['耐光カロル第二楽章']='Carol',['耐闇カロル第二楽章']='Carol',
	['魔物のララバイ']='Lullaby',['魔物のララバイII']='Lullaby',['魔物達のララバイ']='Lullaby',['魔物達のララバイII']='Lullaby',
	['炎のスレノディ']='Threnody',['氷のスレノディ']='Threnody',['風のスレノディ']='Threnody',['土のスレノディ']='Threnody',['雷のスレノディ']='Threnody',['水のスレノディ']='Threnody',['光のスレノディ']='Threnody',['闇のスレノディ']='Threnody',
	['炎のスレノディII']='Threnody',['氷のスレノディII']='Threnody',['風のスレノディII']='Threnody',['土のスレノディII']='Threnody',['雷のスレノディII']='Threnody',['水のスレノディII']='Threnody',['光のスレノディII']='Threnody',['闇のスレノディII']='Threnody',
	['戦場のエレジー']='Elegy',['修羅のエレジー']='Elegy',
	['魔物のレクイエム']='Requiem',['魔物のレクイエムII']='Requiem',['魔物のレクイエムIII']='Requiem',['魔物のレクイエムIV']='Requiem',['魔物のレクイエムV']='Requiem',['魔物のレクイエムVI']='Requiem',['魔物のレクイエムVII']='Requiem',
	
	
	['ファイアマニューバ']='Maneuver',['アイスマニューバ']='Maneuver',['ウィンドマニューバ']='Maneuver',['アースマニューバ']='Maneuver',
	['サンダーマニューバ']='Maneuver',['ウォータマニューバ']='Maneuver',['ライトマニューバ']='Maneuver',['ダークマニューバ']='Maneuver',
	
	['空蝉の術:壱']='Utsusemi',['空蝉の術:弐']='Utsusemi',['空蝉の術:参']='Utsusemi',
	['火遁の術:壱']='ElementalNinjutsu',['水遁の術:壱']='ElementalNinjutsu',['雷遁の術:壱']='ElementalNinjutsu',
	['土遁の術:壱']='ElementalNinjutsu',['風遁の術:壱']='ElementalNinjutsu',['氷遁の術:壱']='ElementalNinjutsu',
	['火遁の術:弐']='ElementalNinjutsu',['水遁の術:弐']='ElementalNinjutsu',['雷遁の術:弐']='ElementalNinjutsu',
	['土遁の術:弐']='ElementalNinjutsu',['風遁の術:弐']='ElementalNinjutsu',['氷遁の術:弐']='ElementalNinjutsu',
	['火遁の術:参']='ElementalNinjutsu',['水遁の術:参']='ElementalNinjutsu',['雷遁の術:参']='ElementalNinjutsu',
	['土遁の術:参']='ElementalNinjutsu',['風遁の術:参']='ElementalNinjutsu',['氷遁の術:参']='ElementalNinjutsu',
	['遁甲の術:壱']='NinjutsuBuff',['遁甲の術:弐'] ='NinjutsuBuff',['物見の術:壱'] ='NinjutsuBuff',
	['妙手の術:壱'] ='NinjutsuBuff',['身替の術:壱'] ='NinjutsuBuff',['月下の術:壱'] ='NinjutsuBuff',['夜陰の術:壱'] ='NinjutsuBuff',['活火の術:壱'] ='NinjutsuBuff',
	['捕縄の術:壱'] ='NinjutsuDebuff',['捕縄の術:弐'] ='NinjutsuDebuff',['暗闇の術:壱'] ='NinjutsuDebuff',['暗闇の術:弐'] ='NinjutsuDebuff',['暗闇の術:参'] ='NinjutsuDebuff',
	['呪縛の術:壱'] ='NinjutsuDebuff',['呪縛の術:弐'] ='NinjutsuDebuff',['毒盛の術:壱'] ='NinjutsuDebuff',['毒盛の術:弐'] ='NinjutsuDebuff',['哀車の術:壱'] ='NinjutsuDeBuff',['幽林の術:壱'] ='NinjutsuDebuff',
	
	-- 契約の履行
	['シアリングライト'] = 'AvatarMagicalPacts',['ルビーの癒し'] = 'AvatarCure',['ポイズンネイル'] = 'AvatarPhysicalPacts',['ルビーの輝き'] = 'AvatarBuffWard',['ルビーの煌き'] = 'AvatarBuffWard',
	['プチメテオ'] = 'AvatarMagicalPacts',['ルビーの癒しII'] = 'AvatarCure',['ホーリーミスト'] = 'AvatarMagicalPacts',['ルビーの安らぎ'] = 'AvatarBuffWard',['ルビーの贖罪'] = 'AvatarBuffWard',
	['アースフューリー'] = 'AvatarMagicalPacts',['ロックスロー'] = 'AvatarPhysicalPacts',['ストーンII'] = 'AvatarMagicalPacts',['ロックバスター'] = 'AvatarPhysicalPacts',['メガリススロー'] = 'AvatarPhysicalPacts',
	['大地の守り'] = 'AvatarBuffWard',['ストーンIV'] = 'AvatarMagicalPacts',['マウンテンバスター'] = 'AvatarPhysicalPacts',['ジオクラッシュ'] = 'AvatarMagicalPacts',['大地の鎧'] = 'AvatarBuffWard',
	['クラッグスロー'] = 'AvatarPhysicalPacts',
	['タイダルウェイブ'] = 'AvatarMagicalPacts',['バラクーダダイブ'] = 'AvatarPhysicalPacts',['ウォータII'] = 'AvatarMagicalPacts',['テールウィップ'] = 'AvatarPhysicalPacts',['スロウガ'] = 'AvatarDeBuffWard',
	['湧水'] = 'AvatarCure',['ウォータIV'] = 'AvatarMagicalPacts',['スピニングダイブ'] = 'AvatarPhysicalPacts',['グランドフォール'] = 'AvatarMagicalPacts',['タイダルロア'] = 'AvatarDeBuffWard',
	['スージングカレント'] = 'AvatarBuffWard',
	['エリアルブラスト'] = 'AvatarMagicalPacts',['クロー'] = 'AvatarPhysicalPacts',['エアロII'] = 'AvatarMagicalPacts',['真空の鎧'] = 'AvatarBuffWard',['風の囁き'] = 'AvatarCure',
	['ヘイスガ'] = 'AvatarBuffWard',['エアロIV'] = 'AvatarMagicalPacts',['プレデタークロー'] = 'AvatarPhysicalPacts',['ウインドブレード'] = 'AvatarMagicalPacts',
	['真空の具足'] = 'AvatarBuffWard',['ヘイスガII'] = 'AvatarBuffWard',
	['インフェルノ'] = 'AvatarMagicalPacts',['パンチ'] = 'AvatarPhysicalPacts',['ファイアII'] = 'AvatarMagicalPacts',['バーニングストライク'] = 'AvatarMagicalPacts',['ダブルパンチ'] = 'AvatarPhysicalPacts',
	['紅蓮の咆哮'] = 'AvatarBuffWard',['ファイアIV'] = 'AvatarMagicalPacts',['フレイムクラッシュ'] = 'AvatarMagicalPacts',['メテオストライク'] = 'AvatarMagicalPacts',['灼熱の咆哮'] = 'AvatarBuffWard',
	['コンフラグストライク'] = 'AvatarMagicalPacts',
	['ダイヤモンドダスト'] = 'AvatarMagicalPacts',['アクスキック'] = 'AvatarPhysicalPacts',['ブリザドII'] = 'AvatarMagicalPacts',['凍てつく鎧'] = 'AvatarBuffWard',['スリプガ'] = 'AvatarDeBuffWard',
	['ダブルスラップ'] = 'AvatarPhysicalPacts',['ブリザドIV'] = 'AvatarMagicalPacts',['ラッシュ'] = 'AvatarPhysicalPacts',['ヘヴンリーストライク'] = 'AvatarMagicalPacts',
	['ダイヤモンドストーム'] = 'AvatarDeBuffWard',['クリスタルブレシング'] = 'AvatarBuffWard',
	['ジャッジボルト'] = 'AvatarMagicalPacts',['ショックストライク'] = 'AvatarPhysicalPacts',['サンダーII'] = 'AvatarMagicalPacts',['サンダースパーク'] = 'AvatarMagicalPacts',['雷鼓'] = 'AvatarBuffWard',
	['雷電の鎧'] = 'AvatarBuffWard',['サンダーIV'] = 'AvatarMagicalPacts',['カオスストライク'] = 'AvatarPhysicalPacts',['サンダーストーム'] = 'AvatarMagicalPacts',
	['スタンガ'] = 'AvatarDeBuffWard',['ボルトストライク'] = 'AvatarPhysicalPacts',
	['ハウリングムーン'] = 'AvatarMagicalPacts',['ムーンリットチャージ'] = 'AvatarPhysicalPacts',['クレセントファング'] = 'AvatarPhysicalPacts',['ルナークライ'] = 'AvatarDeBuffWard',
	['ルナーロア'] = 'AvatarDeBuffWard',['上弦の唸り'] = 'AvatarBuffWard',['下弦の咆哮'] = 'AvatarBuffWard',['エクリプスバイト'] = 'AvatarPhysicalPacts',['ルナーベイ'] = 'AvatarMagicalPacts',
	['ヘヴンズハウル'] = 'AvatarBuffWard',['インパクト'] = 'AvatarMagicalPacts',
	['ルイナスオーメン'] = 'AvatarMagicalPacts',['カミサドー'] = 'AvatarPhysicalPacts',['ソムノレンス'] = 'AvatarDeBuffWard',['ナイトメア'] = 'AvatarDeBuffWard',
	['アルティメットテラー'] = 'AvatarDeBuffWard',['ノクトシールド'] = 'AvatarBuffWard',['ドリームシュラウド'] = 'AvatarBuffWard',['ネザーブラスト'] = 'AvatarMagicalPacts',
	['ナイトテラー'] = 'AvatarMagicalPacts',['パボルノクターナス'] = 'AvatarDeBuffWard',['ブラインドサイド'] = 'AvatarPhysicalPacts',
	['アルタナフェーバー'] = 'AvatarBuffWard',['リーガルスクラッチ'] = 'AvatarPhysicalPacts',['レイズII'] = 'AvatarBuffWard',['ミュインララバイ'] = 'AvatarDeBuffWard',
	['リレイズII'] = 'AvatarBuffWard',['イアリーアイ'] = 'AvatarDeBuffWard',['レベル？ホーリー'] = 'AvatarMagicalPacts',['リーガルガッシュ'] = 'AvatarPhysicalPacts',
	['クラーサクコール'] = 'AvatarMagicalPacts',['ウェルト'] = 'AvatarPhysicalPacts',['ルナティックボイス'] = 'AvatarDeBuffWard',['ラウンドハウス'] = 'AvatarPhysicalPacts',
	['疾風の刃'] = 'AvatarBuffWard',['シヌーク'] = 'AvatarBuffWard',['修羅のエレジー'] = 'AvatarDeBuffWard',['ソニックバフェット'] = 'AvatarMagicalPacts',
	['トルネドII'] = 'AvatarMagicalPacts',['風の守り'] = 'AvatarBuffWard',['ヒステリックアサルト'] = 'AvatarPhysicalPacts',
	['斬鉄剣'] = 'AvatarDeBuffWard',['ディコンストラクション'] = 'AvatarDeBuffWard',['クロノシフト'] = 'AvatarDeBuffWard',['絶対防御'] = 'AvatarBuffWard',
	--物理系青魔法
	['F.リップ']='BluePhysical',['H.バラージ']='BluePhysical',['M.バイト']='BluePhysical',['S.ドライバー']='BluePhysical',['T.アッサルト']='BluePhysical',['まつぼっくり爆弾']='BluePhysical',
	['アシュラクロー']='BluePhysical',['アッパーカット']='BluePhysical',['エンプティスラッシュ']='BluePhysical',['キャノンボール']='BluePhysical',['クアドラストライク']='BluePhysical',	['グランドスラム']='BluePhysical',
	['グルーティナスダート']='BluePhysical',['クローサイクロン']='BluePhysical',['ゴブリンラッシュ']='BluePhysical',['サウリアンスライド']='BluePhysical',	['サドンランジ']='BluePhysical',['サブゼロスマッシュ']='BluePhysical',
	['シードスプレー']='BluePhysical',['ジェットストリーム']='BluePhysical',['シックルスラッシュ']='BluePhysical',	['シンカードリル']='BluePhysical',['スイープガウジ']='BluePhysical',['スパイナルクリーブ']='BluePhysical',
	['スパイラルスピン']='BluePhysical',['スプラウトスマック']='BluePhysical',['ディセバーメント']='BluePhysical',['テールスラップ']='BluePhysical',['テラータッチ']='BluePhysical',['デスシザース']='BluePhysical',
	['デルタスラスト']='BluePhysical',['トゥールビヨン']='BluePhysical',['ハイドロショット']='BluePhysical',['バトルダンス']='BluePhysical',['バニティダイブ']='BluePhysical',['バーチカルクリーヴ']='BluePhysical',
	['パラライズトライアド']='BluePhysical',['パワーアタック']='BluePhysical',['ビルジストーム']='BluePhysical',['フットキック']='BluePhysical',['フライパン']='BluePhysical',['ブラッドレイク']='BluePhysical',
	['ヘッドバット']='BluePhysical',['ヘルダイブ']='BluePhysical',['ベンシクタイフーン']='BluePhysical',['ボディプレス']='BluePhysical',['マヨイタケ']='BluePhysical',['メッタ打ち']='BluePhysical',
	['ラムチャージ']='BluePhysical',['偃月刃']='BluePhysical',['四連突']='BluePhysical',['怒りの一撃']='BluePhysical',['怒りの旋風']='BluePhysical',['槍玉']='BluePhysical',
	['次元殺']='BluePhysical',['種まき']='BluePhysical',['羽根吹雪']='BluePhysical',['重い一撃']='BluePhysical',['ファイナルスピア']='BluePhysical',
	--魔法系青魔法
	['A.バースト']='BlueMagical',['A.ライベーション']='BlueMagical',['B.シュトラール']='BlueMagical',['B.フルゴア']='BlueMagical',['C.ディスチャージ']='BlueMagical',['D.ロア']='BlueMagical',
	['D.ワールウィンド']='BlueMagical',['F.ヒッププレス']='BlueMagical',['F.ホールド']='BlueMagical',['MP吸収キッス']='BlueMagical',['R.デルージュ']='BlueMagical',['R.ブレス']='BlueMagical',
	['T.アップヒーヴ']='BlueMagical',['みんなの怨み']='BlueMagical',['アーフルアイ']='BlueMagical',['アイズオンミー']='BlueMagical',['アイスブレイク']='BlueMagical',['アクリッドストリーム']='BlueMagical',
	['アップルート']='BlueMagical',['アブソルートテラー']='BlueMagical',['アンビルライトニング']='BlueMagical',['ウィンドブレス']='BlueMagical',['ヴェイパースプレー']='BlueMagical',['エントゥーム']='BlueMagical',
	['エンバームアース']='BlueMagical',['オスモーシス']='BlueMagical',['オーロラルドレープ']='BlueMagical',['カオティックアイ']='BlueMagical',['カースドスフィア']='BlueMagical',['ガイストウォール']='BlueMagical',
	['クラッシュサンダー']='BlueMagical',['クルエルジョーク']='BlueMagical',['ゲーツオブハデス']='BlueMagical',['コローシブウーズ']='BlueMagical',['コールドウェーブ']='BlueMagical',['サイレントストーム']='BlueMagical',
	['サウンドブラスト']='BlueMagical',['サブダックション']='BlueMagical',['サペリフィック']='BlueMagical',['サンダーブレス']='BlueMagical',['サンダーボルト']='BlueMagical',
	['サンドスプレー']='BlueMagical',['サーマルパルス']='BlueMagical',['シアリングテンペスト']='BlueMagical',['シープソング']='BlueMagical',['ジェタチュラ']='BlueMagical',['スティンキングガス']='BlueMagical',
	['スカウリングスペイト']='BlueMagical',['スペクトラルフロー']='BlueMagical',['セスプール']='BlueMagical',['ダークオーブ']='BlueMagical',['チャージドホイスカー']='BlueMagical',['ディフュージョンレイ']='BlueMagical',
	['デスレイ']='BlueMagical',['テネブラルクラッシュ']='BlueMagical',['テーリングガスト']='BlueMagical',['テンポラルシフト']='BlueMagical',['ネクターデルージュ']='BlueMagical',['ファイアースピット']='BlueMagical',
	['ファウルウォーター']='BlueMagical',['フェザーティックル']='BlueMagical',['フライトフルロア']='BlueMagical',['ブラッドセイバー']='BlueMagical',['ブランクゲイズ']='BlueMagical',
	['ブリスターローア']='BlueMagical',['ブレーズバウンド']='BlueMagical',['フロストブレス']='BlueMagical',['ヘカトンウェーブ']='BlueMagical',['ベノムシェル']='BlueMagical',['ポーリングサルヴォ']='BlueMagical',
	['ポイズンブレス']='BlueMagical',['ポラーロア']='BlueMagical',['メイルシュトロム']='BlueMagical',['マインドブラスト']='BlueMagical',['モータルレイ']='BlueMagical',['モルトプルメイジ']='BlueMagical',['ヤーン']='BlueMagical',
	['リガージテーション']='BlueMagical',['リービンウィンド']='BlueMagical',['リーフストーム']='BlueMagical',['レテナグレア']='BlueMagical',['レールキャノン']='BlueMagical',['ロウイン']='BlueMagical',
	['吶喊']='BlueMagical',['吸印']='BlueMagical',['吸血']='BlueMagical',['土竜巻']='BlueMagical',['夢想花']='BlueMagical',['水風船爆弾']='BlueMagical',['消化']='BlueMagical',['火炎の息']='BlueMagical',['炸裂弾']='BlueMagical',
	['爆弾投げ']='BlueMagical',['磁鉄粉']='BlueMagical',['神秘の光']='BlueMagical',['自爆']='BlueMagical',['臭い息']='BlueMagical',['贖罪の光']='BlueMagical',['超低周波']='BlueMagical',['針千本']='BlueMagical',
	--回復系青魔法	
	['いやしの風']='BlueHealing',['マジックフルーツ']='BlueHealing',['P.エンブレイス']='BlueHealing',['花粉']='BlueHealing',['レストラル']='BlueHealing',['ホワイトウィンド']='BlueHealing',
	['ワイルドカロット']='BlueHealing',['虚無の風']='BlueHealing',['イグジュビエーション']='BlueHealing',
	--強化系青魔法
	['N.メディテイト']='BlueBuff',['エラチックフラッター']='BlueBuff',['カウンタースタンス']='BlueBuff',['カルカリアンヴァーヴ']='BlueBuff',['コクーン']='BlueBuff',['セイリーンコート']='BlueBuff',['ゼファーマント']='BlueBuff',
	['ねたみ種']='BlueBuff',['バッテリーチャージ']='BlueBuff',['ファンタッド']='BlueBuff',['フェザーバリア']='BlueBuff',['プラズマチャージ']='BlueBuff',['ポーラーブルワーク']='BlueBuff',['マイティガード']='BlueBuff',
	['メメントモーリ']='BlueBuff',['リジェネレーション']='BlueBuff',['リフュエリング']='BlueBuff',['ワームアップ']='BlueBuff',['牙門']='BlueBuff',['共鳴']='BlueBuff',['鯨波']='BlueBuff',['甲羅強化']='BlueBuff',
	['反応炉冷却']='BlueBuff',['金剛身']='BlueBuff',['マジックバリア']='BlueBuff',['メタルボディ']='BlueBuff',['オカルテーション']='BlueBuff',
   }

no_skill_spells_list = S{'リレイズ', 'リレイズII', 'リレイズIII', 'リレイズIV', 'レイズ', 'レイズII', 'レイズIII', 'アレイズ',
                        'トラクタ', 'デジョン', 'デジョンII', 'エスケプ', 'テレポホラ', 'テレポデム', 'テレポメア', 'テレポルテ', 'テレポヨト',
                        'テレポヴァズ', 'リコールパシュ', 'リコールメリファ', 'リコールジャグ', 'リトレース',}


-------------------------------------------------------------------------------------------------------------------
-- Tables to specify general area groupings.  Creates the 'areas' table to be referenced in job files.
-- Zone names provided by world.area/world.zone are currently in all-caps, so defining the same way here.
-------------------------------------------------------------------------------------------------------------------

areas = {}

-- City areas for town gear and behavior.
areas.Cities = S{
    "ル・ルデの庭",
    "ジュノ上層",
    "ジュノ下層",
    "ジュノ港",
    "ウィンダス港",
    "ウィンダス水の区",
    "ウィンダス森の区",
    "ウィンダス石の区",
    "天の塔",
    "サンドリア港",
    "北サンドリア",
    "南サンドリア",
    "バストゥーク港",
    "バストゥーク商業区",
    "バストゥーク鉱山区",
    "大工房",
    "アトルガン白門",
    "タブナジア地下壕",
    "ナシュモ",
    "セルビナ",
    "マウラ",
    "ノーグ",
    "西アドゥリン",
    "東アドゥリン",
    "カザム",
    "ラバオ",
    "チョコボサーキット",
}
-- Adoulin areas, where Ionis will grant special stat bonuses.
areas.Adoulin = S{
    "ヤッセの狩り場",
    "ケイザック古戦場",
    "エヌティエル水林",
    "モリマー台地",
    "ヨルシア森林",
    "ヨルシア森林〔Ｕ〕",
    "シルダス洞窟",
    "シルダス洞窟〔Ｕ〕",
    "マリアミ渓谷マリアミ渓谷",
    "カミール山麓",
    "シィの門",
    "モーの門",
    "ドーの門",
    "ウォーの門",
    "ララ水道",
    "ララ水道〔Ｕ〕",
    "ラ・カザナル宮外郭",
    "ラ・カザナル宮外郭〔Ｕ〕",
    "ラ・カザナル宮内郭",
    "ラ・カザナル宮天守",
}


-------------------------------------------------------------------------------------------------------------------
-- Lists of certain NPCs. (Not up to date)
-------------------------------------------------------------------------------------------------------------------

npcs = {}
npcs.Trust = S{
	'マツイP',
	'モンブロー',
	'アークHM',
	'コーネリア',
	'ユグナス',
	'アークTT',
	'アークMR',
	'アークEV',
	'アークGK	',
	'シャントットII',
	'アシェラII',
	'イロハ',
	'イロハII',
	'ダラクァルン',
	'マッキーチェブキー',
	'キング・オブ・ハーツ',
	'モリマー',
	'テオドール',
	'ウルゴア',
	'ミュモルII',
	'イングリッドII',
	'オーグスト',
	'ロスレーシャ',
	'テンゼンII',
	'プリッシュII',
	'アブクーバ',
	'ナシュメラII',
	'リリゼットII',
	'バラモア',
	'セルテウス',
	'ロベルアクベル',
	'レオノアーヌ',
	'マクシミリアン',
	'カイルパイル',
	'スカリーZ',
	'ロンジェルツ',
	'クポフリート',
	'ミルドリオン',
	'セミ・ラフィーナ',
	'ハルヴァー',
	'ザイドII',
	'ライオンII',
	'ブリジッド',
	'クルタダ',
	'アーデルハイト',
	'アムチュチュ',
	'マヤコフ',
	'アシェラ',
	'チャチャルン',
	'レイ・ランガヴォ',
	'アレヴァト',
	'ギルガメッシュ',
	'クッキーチェブキー',
	'マルグレート',
	'ルガジーン',
	'アベンツィオ',
	'ウェイレア',
	'カラハバルハ',
	'リリゼット',
	'ラーアル',
	'シド',
	'コルモル',	
	'クイン・ハスデンナ',
	'ウカ・トトゥリン',
	'ライニマード',
	'エグセニミルII',
	'クララ',
	'ロマー・ミーゴ',
	'ミュモル',
	'星の神子',
	'エリヴィラ',
	'ノユリ',
	'フェリアスコフィン',
	'ルー・マカラッカ',
	'D.シャントット',
	'モーグリ',
	'アルド',
	'ファブリニクス',
	'マート',
	'ゲッショー',
	'ナジュリス',
	'ルザフ',
	'アヴゼン',
	'メネジン',
	'サクラ',
	'チェルキキ',
	'ナシュメラ',
	'ザザーグ',
	'イングリッド',
	'レコ・ハボッカ',
	'プリッシュ',
	'ウルミア',
	'アイアンイーター',
	'ガダラル',
	'ナジャ・サラヒム',
	'ライオン',
	'テンゼン',
	'ミリ・アリアポー',
	'ヴァレンラール',
	'ヨアヒム',
	'トリオン',
	'フォルカー',
	'アジドマルジド',
	'シャントット',
	'ザイド',
	'エグセニミル',
	'ナジ',
	'クピピ',
	'クリルラ',
	'アヤメ',
	'ナナー・ミーゴ',
	}


